@@ -1,141 +1,62 @@
-# Setting configuration file for Manila services
-# ----------------------------------------------
-# 1) It is possible to set any custom opt to any config group using following:
-# $ export MANILA_OPTGROUP_foo_bar=value
-# where 'foo' is name of config group and 'bar' is name of option.
-#
-# 2) 'MANILA_CONFIGURE_GROUPS' contains list of config group names used to create
-# config groups, but 'MANILA_ENABLED_BACKENDS' is used to set config groups as
-# Manila share back ends. Both can be set like following:
-# $ export MANILA_ENABLED_BACKENDS=foo,bar
-# where 'foo' and 'bar' are names of config groups with opts for some share
-# drivers. By default they are equal. Also be attentive, if you modify both,
-# make sure 'MANILA_CONFIGURE_GROUPS' contains all values from
-# 'MANILA_ENABLED_BACKENDS'.
-# DEFAULT group is always defined, no need to specify it within 'MANILA_CONFIGURE_GROUPS'.
-#
-# 3) Two default backends are used for compatibility with previous approach.
-# They have same configuration except name of backend. Both use generic driver.
-# They can be enabled by adding values of following env vars:
-# 'MANILA_BACKEND1_CONFIG_GROUP_NAME' and 'MANILA_BACKEND2_CONFIG_GROUP_NAME'
-# to the env var 'MANILA_ENABLED_BACKENDS' or will be enabled
-# if 'MANILA_ENABLED_BACKENDS' is empty.
+# Plugin file for enabling manila services
+# ----------------------------------------
 
 # Save trace setting
 XTRACE=$(set +o | grep xtrace)
 set -o xtrace
 
-# Defaults
-# --------
-
-MANILA_GIT_BASE=${MANILA_GIT_BASE:-https://github.com}
-MANILA_REPO_ROOT=${MANILA_REPO_ROOT:-openstack}
-
-MANILACLIENT_REPO=${MANILA_GIT_BASE}/${MANILA_REPO_ROOT}/python-manilaclient.git
-MANILACLIENT_BRANCH=${MANILACLIENT_BRANCH:-master}
-
-MANILA_UI_REPO=${MANILA_GIT_BASE}/${MANILA_REPO_ROOT}/manila-ui.git
-MANILA_UI_BRANCH=${MANILA_UI_BRANCH:-$MANILACLIENT_BRANCH}
-MANILA_UI_ENABLED=$(trueorfalse True MANILA_UI_ENABLED)
-
-# set up default directories
-MANILA_DIR=${MANILA_DIR:=$DEST/manila}
-MANILA_LOCK_PATH=${MANILA_LOCK_PATH:=$OSLO_LOCK_PATH}
-MANILA_LOCK_PATH=${MANILA_LOCK_PATH:=$MANILA_DIR/manila_locks}
-MANILACLIENT_DIR=${MANILACLIENT_DIR:=$DEST/python-manilaclient}
-MANILA_UI_DIR=${MANILA_UI_DIR:=$DEST/manila-ui}
-MANILA_STATE_PATH=${MANILA_STATE_PATH:=$DATA_DIR/manila}
-MANILA_AUTH_CACHE_DIR=${MANILA_AUTH_CACHE_DIR:-/var/cache/manila}
-
-MANILA_CONF_DIR=${MANILA_CONF_DIR:-/etc/manila}
-MANILA_CONF=$MANILA_CONF_DIR/manila.conf
-MANILA_API_PASTE_INI=$MANILA_CONF_DIR/api-paste.ini
-
-MANILA_DEFAULT_SHARE_TYPE=${MANILA_DEFAULT_SHARE_TYPE:-default}
-# MANILA_DEFAULT_SHARE_TYPE_EXTRA_SPECS is expected to contain extra specs key-value pairs,
-# that should be assigned to default share type. Both - qualified and unqualified extra specs are supported.
-# Pairs are separated by spaces, value is assigned to key using sign of equality. Examples:
-# MANILA_DEFAULT_SHARE_TYPE_EXTRA_SPECS='foo=bar'
-# MANILA_DEFAULT_SHARE_TYPE_EXTRA_SPECS='foo=bar quuz=xyzzy'
-# MANILA_DEFAULT_SHARE_TYPE_EXTRA_SPECS='foo=bar quuz=xyzzy fakeprefix:baz=waldo'
-MANILA_DEFAULT_SHARE_TYPE_EXTRA_SPECS=${MANILA_DEFAULT_SHARE_TYPE_EXTRA_SPECS:-''}
-
-# Public facing bits
-MANILA_SERVICE_HOST=${MANILA_SERVICE_HOST:-$SERVICE_HOST}
-MANILA_SERVICE_PORT=${MANILA_SERVICE_PORT:-8786}
-MANILA_SERVICE_PORT_INT=${MANILA_SERVICE_PORT_INT:-18776}
-MANILA_SERVICE_PROTOCOL=${MANILA_SERVICE_PROTOCOL:-$SERVICE_PROTOCOL}
-
-# Support entry points installation of console scripts
-if [[ -d $MANILA_DIR/bin ]]; then
-    MANILA_BIN_DIR=$MANILA_DIR/bin
-else
-    MANILA_BIN_DIR=$(get_python_exec_prefix)
-fi
-
-# Common opts
-SHARE_NAME_PREFIX=${SHARE_NAME_PREFIX:-share-}
-MANILA_ENABLED_SHARE_PROTOCOLS=${ENABLED_SHARE_PROTOCOLS:-"NFS,CIFS"}
-MANILA_SCHEDULER_DRIVER=${MANILA_SCHEDULER_DRIVER:-manila.scheduler.filter_scheduler.FilterScheduler}
-MANILA_SERVICE_SECGROUP="manila-service"
-
-# Following env var defines whether to apply downgrade migrations setting up DB or not.
-# If it is set to False, then only 'upgrade' migrations will be applied.
-# If it is set to True, then will be applied 'upgrade', 'downgrade' and 'upgrade'
-# migrations again.
-MANILA_USE_DOWNGRADE_MIGRATIONS=${MANILA_USE_DOWNGRADE_MIGRATIONS:-"False"}
-
-# Common info for Generic driver(s)
-SHARE_DRIVER=${SHARE_DRIVER:-manila.share.drivers.generic.GenericShareDriver}
-
-eval USER_HOME=~
-MANILA_PATH_TO_PUBLIC_KEY=${MANILA_PATH_TO_PUBLIC_KEY:-"$USER_HOME/.ssh/id_rsa.pub"}
-MANILA_PATH_TO_PRIVATE_KEY=${MANILA_PATH_TO_PRIVATE_KEY:-"$USER_HOME/.ssh/id_rsa"}
-MANILA_SERVICE_KEYPAIR_NAME=${MANILA_SERVICE_KEYPAIR_NAME:-"manila-service"}
-
-MANILA_SERVICE_INSTANCE_USER=${MANILA_SERVICE_INSTANCE_USER:-"manila"}
-MANILA_SERVICE_IMAGE_URL=${MANILA_SERVICE_IMAGE_URL:-"https://github.com/uglide/manila-image-elements/releases/download/0.1.0/manila-service-image.qcow2"}
-MANILA_SERVICE_IMAGE_NAME=${MANILA_SERVICE_IMAGE_NAME:-"manila-service-image"}
-# Third party CI Vendors should set this to false to skip the service image download
-MANILA_SERVICE_IMAGE_ENABLED=$(trueorfalse True MANILA_SERVICE_IMAGE_ENABLED)
-
-MANILA_USE_SERVICE_INSTANCE_PASSWORD=${MANILA_USE_SERVICE_INSTANCE_PASSWORD:-"False"}
-MANILA_SERVICE_INSTANCE_PASSWORD=${MANILA_SERVICE_INSTANCE_PASSWORD:-"manila"}
-
-MANILA_SERVICE_VM_FLAVOR_REF=${MANILA_SERVICE_VM_FLAVOR_REF:-100}
-MANILA_SERVICE_VM_FLAVOR_NAME=${MANILA_SERVICE_VM_FLAVOR_NAME:-"manila-service-flavor"}
-MANILA_SERVICE_VM_FLAVOR_RAM=${MANILA_SERVICE_VM_FLAVOR_RAM:-128}
-MANILA_SERVICE_VM_FLAVOR_DISK=${MANILA_SERVICE_VM_FLAVOR_DISK:-0}
-MANILA_SERVICE_VM_FLAVOR_VCPUS=${MANILA_SERVICE_VM_FLAVOR_VCPUS:-1}
-
-# Support for multi backend configuration (default is no support)
-MANILA_MULTI_BACKEND=$(trueorfalse False MANILA_MULTI_BACKEND)
-DEPRECATED_TEXT="$DEPRECATED_TEXT\n'MANILA_MULTI_BACKEND' is deprecated,
-    it makes influence only when is set to True and 'MANILA_ENABLED_BACKENDS' is not set.
-    Use 'MANILA_ENABLED_BACKENDS' instead if you want to use custom setting.
-    Set there a list of back end names to be enabled.\n
-    To configure custom back ends use (any opt in any group can be set in this way) following:
-    MANILA_OPTGROUP_foo_bar=value
-    where 'foo' is name of config group and 'bar' is name of option.\n"
-
-# First share backend data, that will be used in any installation
-MANILA_BACKEND1_CONFIG_GROUP_NAME=${MANILA_BACKEND1_CONFIG_GROUP_NAME:-generic1}  # deprecated
-MANILA_SHARE_BACKEND1_NAME=${MANILA_SHARE_BACKEND1_NAME:-GENERIC1}  # deprecated
-
-# Second share backend data, that will be used only with MANILA_MULTI_BACKEND=True
-MANILA_BACKEND2_CONFIG_GROUP_NAME=${MANILA_BACKEND2_CONFIG_GROUP_NAME:-generic2}  # deprecated
-MANILA_SHARE_BACKEND2_NAME=${MANILA_SHARE_BACKEND2_NAME:-GENERIC2}  # deprecated
-
-
 # Entry Points
 # ------------
 
+function _clean_share_group {
+    local vg=$1
+    local vg_prefix=$2
+    # Clean out existing shares
+    for lv in `sudo lvs --noheadings -o lv_name $vg`; do
+        # vg_prefix prefixes the LVs we want
+        if [[ "${lv#$vg_prefix}" != "$lv" ]]; then
+            sudo umount -f $MANILA_MNT_DIR/$lv
+            sudo lvremove -f $vg/$lv
+            sudo rm -rf $MANILA_MNT_DIR/$lv
+        fi
+    done
+}
+
+function _clean_manila_lvm_backing_file {
+    local vg=$1
+
+    # if there is no logical volume left, it's safe to attempt a cleanup
+    # of the backing file
+    if [ -z "`sudo lvs --noheadings -o lv_name $vg`" ]; then
+        # if the backing physical device is a loop device, it was probably setup by devstack
+        VG_DEV=$(sudo losetup -j $DATA_DIR/${vg}-backing-file | awk -F':' '/backing-file/ { print $1
+}')
+        if [[ -n "$VG_DEV" ]]; then
+            sudo losetup -d $VG_DEV
+            rm -f $DATA_DIR/${vg}-backing-file
+        fi
+    fi
+}
+
+function _clean_zfsonlinux_data {
+    for filename in "$MANILA_ZFSONLINUX_BACKEND_FILES_CONTAINER_DIR"/*; do
+        if [[ $(sudo zpool list | grep $filename) ]]; then
+            echo "Destroying zpool named $filename"
+            sudo zpool destroy -f $filename
+            file="$MANILA_ZFSONLINUX_BACKEND_FILES_CONTAINER_DIR$filename"
+            echo "Destroying file named $file"
+            rm -f $file
+        fi
+    done
+}
+
 # cleanup_manila - Remove residual data files, anything left over from previous
 # runs that a clean run would need to clean up
 function cleanup_manila {
-    # This is placeholder.
-    # All stuff, that are created by Generic driver will be cleaned up by other services.
-    :
+    # All stuff, that are created by share drivers will be cleaned up by other services.
+    _clean_share_group $SHARE_GROUP $SHARE_NAME_PREFIX
+    _clean_manila_lvm_backing_file $SHARE_GROUP
+    _clean_zfsonlinux_data
 }
 
 # configure_default_backends - configures default Manila backends with generic driver.
@@ -244,15 +165,10 @@ function configure_manila {
     # Remove old conf file if exists
     rm -f $MANILA_CONF
 
-    iniset $MANILA_CONF keystone_authtoken identity_uri $KEYSTONE_AUTH_URI
-    iniset $MANILA_CONF keystone_authtoken admin_tenant_name $SERVICE_TENANT_NAME
-    iniset $MANILA_CONF keystone_authtoken admin_user manila
-    iniset $MANILA_CONF keystone_authtoken admin_password $SERVICE_PASSWORD
-    iniset $MANILA_CONF keystone_authtoken signing_dir $MANILA_AUTH_CACHE_DIR
+    configure_auth_token_middleware $MANILA_CONF manila  $MANILA_AUTH_CACHE_DIR
 
     iniset $MANILA_CONF DEFAULT auth_strategy keystone
     iniset $MANILA_CONF DEFAULT debug True
-    iniset $MANILA_CONF DEFAULT verbose True
     iniset $MANILA_CONF DEFAULT scheduler_driver $MANILA_SCHEDULER_DRIVER
     iniset $MANILA_CONF DEFAULT share_name_template ${SHARE_NAME_PREFIX}%s
     iniset $MANILA_CONF DATABASE connection `database_connection_url manila`
@@ -263,16 +179,27 @@ function configure_manila {
     iniset $MANILA_CONF DEFAULT state_path $MANILA_STATE_PATH
     iniset $MANILA_CONF DEFAULT default_share_type $MANILA_DEFAULT_SHARE_TYPE
 
-    iniset $MANILA_CONF DEFAULT nova_admin_password $SERVICE_PASSWORD
-    iniset $MANILA_CONF DEFAULT cinder_admin_password $SERVICE_PASSWORD
-    iniset $MANILA_CONF DEFAULT neutron_admin_password $SERVICE_PASSWORD
-
     iniset $MANILA_CONF DEFAULT enabled_share_protocols $MANILA_ENABLED_SHARE_PROTOCOLS
 
     iniset $MANILA_CONF oslo_concurrency lock_path $MANILA_LOCK_PATH
 
     iniset $MANILA_CONF DEFAULT wsgi_keep_alive False
 
+    iniset $MANILA_CONF DEFAULT lvm_share_volume_group $SHARE_GROUP
+
+    # Set the replica_state_update_interval
+    iniset $MANILA_CONF DEFAULT replica_state_update_interval $MANILA_REPLICA_STATE_UPDATE_INTERVAL
+
+    if is_service_enabled neutron; then
+        configure_auth_token_middleware $MANILA_CONF neutron $MANILA_AUTH_CACHE_DIR neutron
+    fi
+    if is_service_enabled nova; then
+        configure_auth_token_middleware $MANILA_CONF nova $MANILA_AUTH_CACHE_DIR nova
+    fi
+    if is_service_enabled cinder; then
+        configure_auth_token_middleware $MANILA_CONF cinder $MANILA_AUTH_CACHE_DIR cinder
+    fi
+
     # Note: set up config group does not mean that this backend will be enabled.
     # To enable it, specify its name explicitly using "enabled_share_backends" opt.
     configure_default_backends
@@ -318,17 +245,22 @@ function configure_manila {
     MANILA_CONFIGURE_GROUPS=${MANILA_CONFIGURE_GROUPS:-"$MANILA_ENABLED_BACKENDS"}
     set_config_opts $MANILA_CONFIGURE_GROUPS
     set_config_opts DEFAULT
-
-    if is_service_enabled horizon && [ "$MANILA_UI_ENABLED" = "True" ]; then
-        configure_manila_ui
-    fi
 }
 
 
 function configure_manila_ui {
-    setup_develop $MANILA_UI_DIR
+    if is_service_enabled horizon && [ "$MANILA_UI_ENABLED" = "True" ]; then
+        # NOTE(vponomaryov): workaround for devstack bug: 1540328
+        # where devstack install 'test-requirements' but should not do it
+        # for manila-ui project as it installs Horizon from url.
+        # Remove following two 'mv' commands when mentioned bug is fixed.
+        mv $MANILA_UI_DIR/test-requirements.txt $MANILA_UI_DIR/_test-requirements.txt
 
-    cp $MANILA_UI_DIR/manila_ui/enabled/_90_manila_*.py $HORIZON_DIR/openstack_dashboard/local/enabled
+        setup_develop $MANILA_UI_DIR
+        cp $MANILA_UI_DIR/manila_ui/enabled/_90_manila_*.py $HORIZON_DIR/openstack_dashboard/local/enabled
+
+        mv $MANILA_UI_DIR/_test-requirements.txt $MANILA_UI_DIR/test-requirements.txt
+    fi
 }
 
 
@@ -341,29 +273,55 @@ function create_manila_service_keypair {
 # driver, and only if it is configured to mode without handling of share servers.
 function create_service_share_servers {
     private_net_id=$(nova net-list | grep ' private ' | get_field 1)
+    created_admin_network=false
     for BE in ${MANILA_ENABLED_BACKENDS//,/ }; do
         driver_handles_share_servers=$(iniget $MANILA_CONF $BE driver_handles_share_servers)
         share_driver=$(iniget $MANILA_CONF $BE share_driver)
         generic_driver='manila.share.drivers.generic.GenericShareDriver'
-        if [[ $(trueorfalse False driver_handles_share_servers) == False && $share_driver == $generic_driver ]]; then
-            vm_name='manila_service_share_server_'$BE
-            nova boot $vm_name \
-                --flavor $MANILA_SERVICE_VM_FLAVOR_NAME \
-                --image $MANILA_SERVICE_IMAGE_NAME \
-                --nic net-id=$private_net_id \
-                --security-groups $MANILA_SERVICE_SECGROUP \
-                --key-name $MANILA_SERVICE_KEYPAIR_NAME
+        if [[ $share_driver == $generic_driver ]]; then
+            if [[ $(trueorfalse False driver_handles_share_servers) == False ]]; then
+                vm_name='manila_service_share_server_'$BE
+                nova boot $vm_name \
+                    --flavor $MANILA_SERVICE_VM_FLAVOR_NAME \
+                    --image $MANILA_SERVICE_IMAGE_NAME \
+                    --nic net-id=$private_net_id \
+                    --security-groups $MANILA_SERVICE_SECGROUP \
+                    --key-name $MANILA_SERVICE_KEYPAIR_NAME
 
-            vm_id=$(nova show $vm_name | grep ' id ' | get_field 2)
+                vm_id=$(nova show $vm_name | grep ' id ' | get_field 2)
 
-            iniset $MANILA_CONF $BE service_instance_name_or_id $vm_id
-            iniset $MANILA_CONF $BE service_net_name_or_ip private
-            iniset $MANILA_CONF $BE tenant_net_name_or_ip private
-            iniset $MANILA_CONF $BE migration_data_copy_node_ip $PUBLIC_NETWORK_GATEWAY
+                iniset $MANILA_CONF $BE service_instance_name_or_id $vm_id
+                iniset $MANILA_CONF $BE service_net_name_or_ip private
+                iniset $MANILA_CONF $BE tenant_net_name_or_ip private
+            else
+                if is_service_enabled neutron; then
+                    if [ $created_admin_network == false ]; then
+                        admin_net_id=$(neutron net-create --tenant-id $TENANT_ID admin_net | grep ' id ' | get_field 2)
+                        admin_subnet_id=$(neutron subnet-create --tenant-id $TENANT_ID --ip_version 4 --no-gateway --name admin_subnet --subnetpool None $admin_net_id $FIXED_RANGE | grep ' id ' | get_field 2)
+                        created_admin_network=true
+                    fi
+                    iniset $MANILA_CONF $BE admin_network_id $admin_net_id
+                    iniset $MANILA_CONF $BE admin_subnet_id $admin_subnet_id
+                fi
+            fi
         fi
     done
+    configure_data_service_generic_driver
 }
 
+function configure_data_service_generic_driver {
+    enabled_backends=(${MANILA_ENABLED_BACKENDS//,/ })
+    share_driver=$(iniget $MANILA_CONF ${enabled_backends[0]} share_driver)
+    generic_driver='manila.share.drivers.generic.GenericShareDriver'
+    if [[ $share_driver == $generic_driver ]]; then
+        driver_handles_share_servers=$(iniget $MANILA_CONF ${enabled_backends[0]} driver_handles_share_servers)
+        if [[ $(trueorfalse False driver_handles_share_servers) == False ]]; then
+            iniset $MANILA_CONF DEFAULT data_node_access_ip $PUBLIC_NETWORK_GATEWAY
+        else
+            iniset $MANILA_CONF DEFAULT data_node_access_ip $FIXED_RANGE
+        fi
+    fi
+}
 # create_manila_service_flavor - creates flavor, that will be used by backends
 # with configured generic driver to boot Nova VMs with.
 function create_manila_service_flavor {
@@ -461,23 +419,9 @@ function create_default_share_type {
     enabled_backends=(${MANILA_ENABLED_BACKENDS//,/ })
     driver_handles_share_servers=$(iniget $MANILA_CONF ${enabled_backends[0]} driver_handles_share_servers)
 
-    manila \
-        --debug \
-        --os-auth-url $KEYSTONE_AUTH_URI/v2.0 \
-        --os-tenant-name ${OS_PROJECT_NAME:-$OS_TENANT_NAME} \
-        --os-username $OS_USERNAME \
-        --os-password $OS_PASSWORD \
-        --os-region-name $OS_REGION_NAME \
-        type-create $MANILA_DEFAULT_SHARE_TYPE $driver_handles_share_servers
+    manila type-create $MANILA_DEFAULT_SHARE_TYPE $driver_handles_share_servers
     if [[ $MANILA_DEFAULT_SHARE_TYPE_EXTRA_SPECS ]]; then
-        manila \
-            --debug \
-            --os-auth-url $KEYSTONE_AUTH_URI/v2.0 \
-            --os-tenant-name ${OS_PROJECT_NAME:-$OS_TENANT_NAME} \
-            --os-username $OS_USERNAME \
-            --os-password $OS_PASSWORD \
-            --os-region-name $OS_REGION_NAME \
-            type-key $MANILA_DEFAULT_SHARE_TYPE set $MANILA_DEFAULT_SHARE_TYPE_EXTRA_SPECS
+        manila type-key $MANILA_DEFAULT_SHARE_TYPE set $MANILA_DEFAULT_SHARE_TYPE_EXTRA_SPECS
     fi
 }
 
@@ -487,7 +431,7 @@ function init_manila {
 
     if is_service_enabled $DATABASE_BACKENDS; then
         # (re)create manila database
-        recreate_database manila utf8
+        recreate_database manila
 
         $MANILA_BIN_DIR/manila-manage db sync
 
@@ -502,6 +446,83 @@ function init_manila {
         $MANILA_BIN_DIR/manila-manage db version
     fi
 
+    if [ "$SHARE_DRIVER" == "manila.share.drivers.lvm.LVMShareDriver" ]; then
+        if is_service_enabled m-shr; then
+            # Configure a default volume group called '`lvm-shares`' for the share
+            # service if it does not yet exist.  If you don't wish to use a file backed
+            # volume group, create your own volume group called ``stack-volumes`` before
+            # invoking ``stack.sh``.
+            #
+            # By default, the backing file is 8G in size, and is stored in ``/opt/stack/data``.
+
+            if ! sudo vgs $SHARE_GROUP; then
+                if [ "$CONFIGURE_BACKING_FILE" = "True" ]; then
+                    SHARE_BACKING_FILE=${SHARE_BACKING_FILE:-$DATA_DIR/${SHARE_GROUP}-backing-file}
+                    # Only create if the file doesn't already exists
+                    [[ -f $SHARE_BACKING_FILE ]] || truncate -s $SHARE_BACKING_FILE_SIZE $SHARE_BACKING_FILE
+                    DEV=`sudo losetup -f --show $SHARE_BACKING_FILE`
+                else
+                    DEV=$SHARE_BACKING_FILE
+                fi
+                # Only create if the loopback device doesn't contain $SHARE_GROUP
+                if ! sudo vgs $SHARE_GROUP; then sudo vgcreate $SHARE_GROUP $DEV; fi
+            fi
+
+            mkdir -p $MANILA_STATE_PATH/shares
+        fi
+    elif [ "$SHARE_DRIVER" == "manila.share.drivers.zfsonlinux.driver.ZFSonLinuxShareDriver" ]; then
+        if is_service_enabled m-shr; then
+            mkdir -p $MANILA_ZFSONLINUX_BACKEND_FILES_CONTAINER_DIR
+            file_counter=0
+            for BE in ${MANILA_ENABLED_BACKENDS//,/ }; do
+                if [[ $file_counter == 0 ]]; then
+                    # NOTE(vponomaryov): create two pools for first ZFS backend
+                    # to cover different use cases that are supported by driver:
+                    # - Support of more than one zpool for share backend.
+                    # - Support of nested datasets.
+                    local first_file="$MANILA_ZFSONLINUX_BACKEND_FILES_CONTAINER_DIR"/alpha
+                    local second_file="$MANILA_ZFSONLINUX_BACKEND_FILES_CONTAINER_DIR"/betta
+                    truncate -s $MANILA_ZFSONLINUX_ZPOOL_SIZE $first_file
+                    truncate -s $MANILA_ZFSONLINUX_ZPOOL_SIZE $second_file
+                    sudo zpool create alpha $first_file
+                    sudo zpool create betta $second_file
+                    # Create subdir (nested dataset) for second pool
+                    sudo zfs create betta/subdir
+                    iniset $MANILA_CONF $BE zfs_zpool_list alpha,betta/subdir
+                elif [[ $file_counter == 1 ]]; then
+                    local file="$MANILA_ZFSONLINUX_BACKEND_FILES_CONTAINER_DIR"/gamma
+                    truncate -s $MANILA_ZFSONLINUX_ZPOOL_SIZE $file
+                    sudo zpool create gamma $file
+                    iniset $MANILA_CONF $BE zfs_zpool_list gamma
+                else
+                    local filename=file"$file_counter"
+                    local file="$MANILA_ZFSONLINUX_BACKEND_FILES_CONTAINER_DIR"/"$filename"
+                    truncate -s $MANILA_ZFSONLINUX_ZPOOL_SIZE $file
+                    sudo zpool create $filename $file
+                    iniset $MANILA_CONF $BE zfs_zpool_list $filename
+                fi
+                iniset $MANILA_CONF $BE zfs_share_export_ip $MANILA_ZFSONLINUX_SHARE_EXPORT_IP
+                iniset $MANILA_CONF $BE zfs_service_ip $MANILA_ZFSONLINUX_SERVICE_IP
+                iniset $MANILA_CONF $BE zfs_dataset_creation_options $MANILA_ZFSONLINUX_DATASET_CREATION_OPTIONS
+                iniset $MANILA_CONF $BE zfs_use_ssh $MANILA_ZFSONLINUX_USE_SSH
+                iniset $MANILA_CONF $BE zfs_ssh_username $MANILA_ZFSONLINUX_SSH_USERNAME
+                iniset $MANILA_CONF $BE replication_domain $MANILA_ZFSONLINUX_REPLICATION_DOMAIN
+                let "file_counter=file_counter+1"
+            done
+            # Install the server's SSH key in our known_hosts file
+            eval STACK_HOME=~$STACK_USER
+            ssh-keyscan ${MANILA_ZFSONLINUX_SERVICE_IP} >> $STACK_HOME/.ssh/known_hosts
+            # If the server is this machine, setup trust for ourselves (otherwise you're on your own)
+            if [ "$MANILA_ZFSONLINUX_SERVICE_IP" = "127.0.0.1" ] || [ "$MANILA_ZFSONLINUX_SERVICE_IP" = "localhost" ] ; then
+                # Trust our own SSH keys
+                eval SSH_USER_HOME=~$MANILA_ZFSONLINUX_SSH_USERNAME
+                cat $STACK_HOME/.ssh/*.pub >> $SSH_USER_HOME/.ssh/authorized_keys
+                # Give ssh user sudo access
+                echo "$MANILA_ZFSONLINUX_SSH_USERNAME ALL=(ALL) NOPASSWD: ALL" | sudo tee -a /etc/sudoers > /dev/null
+            fi
+        fi
+    fi
+
     # Create cache dir
     sudo mkdir -p $MANILA_AUTH_CACHE_DIR
     sudo chown $STACK_USER $MANILA_AUTH_CACHE_DIR
@@ -512,12 +533,72 @@ function init_manila {
 function install_manila {
     git_clone $MANILACLIENT_REPO $MANILACLIENT_DIR $MANILACLIENT_BRANCH
 
+    if [ "$SHARE_DRIVER" == "manila.share.drivers.lvm.LVMShareDriver" ]; then
+        if is_service_enabled m-shr; then
+            if is_ubuntu; then
+                sudo apt-get install -y nfs-kernel-server nfs-common samba
+            elif is_fedora; then
+                sudo yum install -y nfs-utils nfs-utils-lib samba
+            fi
+        fi
+    elif [ "$SHARE_DRIVER" == "manila.share.drivers.zfsonlinux.driver.ZFSonLinuxShareDriver" ]; then
+        if is_service_enabled m-shr; then
+            if is_ubuntu; then
+                sudo apt-get install -y nfs-kernel-server nfs-common samba
+                # NOTE(vponomaryov): following installation is valid for Ubuntu 'trusty'.
+                sudo apt-get install -y software-properties-common
+                sudo apt-add-repository --yes ppa:zfs-native/stable
+                sudo apt-get -y -q update && sudo apt-get -y -q upgrade
+                sudo apt-get install -y linux-headers-generic
+                sudo apt-get install -y build-essential
+                sudo apt-get install -y ubuntu-zfs
+                sudo modprobe zfs
+            else
+                echo "Manila Devstack plugin does not support installation "\
+                    "of ZFS packages for non-'Ubuntu-trusty' distros. "\
+                    "Please, install it first by other means or add its support "\
+                    "for your distro."
+                exit 1
+            fi
+        fi
+    fi
+
     # install manila-ui if horizon is enabled
     if is_service_enabled horizon && [ "$MANILA_UI_ENABLED" = "True" ]; then
         git_clone $MANILA_UI_REPO $MANILA_UI_DIR $MANILA_UI_BRANCH
     fi
 }
 
+#configure_samba - Configure node as Samba server
+function configure_samba {
+    if [ "$SHARE_DRIVER" == "manila.share.drivers.lvm.LVMShareDriver" ]; then
+        # TODO(vponomaryov): add here condition for ZFSonLinux driver too
+        # when it starts to support SAMBA
+        samba_daemon_name=smbd
+        if is_service_enabled m-shr; then
+            if is_fedora; then
+                samba_daemon_name=smb
+            fi
+            sudo service $samba_daemon_name restart || echo "Couldn't restart '$samba_daemon_name' service"
+        fi
+
+        sudo cp /usr/share/samba/smb.conf $SMB_CONF
+        sudo chown $STACK_USER -R /etc/samba
+        iniset $SMB_CONF global include registry
+        iniset $SMB_CONF global security user
+        if [ ! -d "$SMB_PRIVATE_DIR" ]; then
+            sudo mkdir $SMB_PRIVATE_DIR
+            sudo touch $SMB_PRIVATE_DIR/secrets.tdb
+        fi
+
+        for backend_name in ${MANILA_ENABLED_BACKENDS//,/ }; do
+            iniset $MANILA_CONF $backend_name driver_handles_share_servers False
+            iniset $MANILA_CONF $backend_name lvm_share_export_ip $HOST_IP
+        done
+        iniset $MANILA_CONF DEFAULT data_node_access_ip $HOST_IP
+    fi
+}
+
 # start_manila - Start running processes, including screen
 function start_manila {
     # restart apache to reload running horizon if manila-ui is enabled
@@ -529,6 +610,7 @@ function start_manila {
     screen_it m-api "cd $MANILA_DIR && $MANILA_BIN_DIR/manila-api --config-file $MANILA_CONF"
     screen_it m-shr "cd $MANILA_DIR && $MANILA_BIN_DIR/manila-share --config-file $MANILA_CONF"
     screen_it m-sch "cd $MANILA_DIR && $MANILA_BIN_DIR/manila-scheduler --config-file $MANILA_CONF"
+    screen_it m-dat "cd $MANILA_DIR && $MANILA_BIN_DIR/manila-data --config-file $MANILA_CONF"
 
     # Start proxies if enabled
     if is_service_enabled tls-proxy; then
@@ -539,7 +621,7 @@ function start_manila {
 # stop_manila - Stop running processes
 function stop_manila {
     # Kill the manila screen windows
-    for serv in m-api m-sch m-shr; do
+    for serv in m-api m-sch m-shr m-dat; do
         screen -S $SCREEN_NAME -p $serv -X kill
     done
 }
@@ -550,6 +632,7 @@ function update_tempest {
         if [ $(trueorfalse False MANILA_USE_SERVICE_INSTANCE_PASSWORD) == True ]; then
             iniset $TEMPEST_DIR/etc/tempest.conf share image_password $MANILA_SERVICE_INSTANCE_PASSWORD
         fi
+        iniset $TEMPEST_DIR/etc/tempest.conf share image_with_share_tools $MANILA_SERVICE_IMAGE_NAME
     fi
 }
 
@@ -577,15 +660,32 @@ elif [[ "$1" == "stack" && "$2" == "post-config" ]]; then
     init_manila
     echo_summary "Installing extra libraries"
     install_libraries
+
+    # Neutron config update
+    if is_service_enabled neutron; then
+        iniset $Q_DHCP_CONF_FILE DEFAULT dnsmasq_local_resolv False
+    fi
+
+    # Cinder config update
+    if is_service_enabled cinder && [[ -n "$CINDER_OVERSUBSCRIPTION_RATIO" ]]; then
+        CINDER_CONF=${CINDER_CONF:-/etc/cinder/cinder.conf}
+        CINDER_ENABLED_BACKENDS=$(iniget $CINDER_CONF DEFAULT enabled_backends)
+        for BN in ${CINDER_ENABLED_BACKENDS//,/ }; do
+            iniset $CINDER_CONF $BN lvm_max_over_subscription_ratio $CINDER_OVERSUBSCRIPTION_RATIO
+        done
+        iniset $CINDER_CONF DEFAULT max_over_subscription_ratio $CINDER_OVERSUBSCRIPTION_RATIO
+    fi
 elif [[ "$1" == "stack" && "$2" == "extra" ]]; then
     echo_summary "Creating Manila entities for auth service"
     create_manila_accounts
 
-    echo_summary "Creating Manila service flavor"
-    create_manila_service_flavor
+    if is_service_enabled nova; then
+        echo_summary "Creating Manila service flavor"
+        create_manila_service_flavor
 
-    echo_summary "Creating Manila service security group"
-    create_manila_service_secgroup
+        echo_summary "Creating Manila service security group"
+        create_manila_service_secgroup
+    fi
 
     # Skip image downloads when disabled.
     # This way vendor Manila driver CI tests can skip
@@ -597,12 +697,17 @@ elif [[ "$1" == "stack" && "$2" == "extra" ]]; then
         echo_summary "Skipping download of Manila service image"
     fi
 
-    echo_summary "Creating Manila service keypair"
-    create_manila_service_keypair
+    if is_service_enabled nova; then
+        echo_summary "Creating Manila service keypair"
+        create_manila_service_keypair
 
-    echo_summary "Creating Manila service VMs for generic driver \
-        backends for which handlng of share servers is disabled."
-    create_service_share_servers
+        echo_summary "Creating Manila service VMs for generic driver \
+            backends for which handlng of share servers is disabled."
+        create_service_share_servers
+    fi
+
+    echo_summary "Configure Samba server"
+    configure_samba
 
     echo_summary "Starting Manila"
     start_manila
@@ -612,6 +717,9 @@ elif [[ "$1" == "stack" && "$2" == "extra" ]]; then
 
     echo_summary "Update Tempest config"
     update_tempest
+
+    echo_summary "Configuring Manila UI"
+    configure_manila_ui
 fi
 
 if [[ "$1" == "unstack" ]]; then
