@@ -16,29 +16,29 @@
 #    under the License.
 
 """
-WSGI middleware for OpenStack Share API.
+WSGI middleware for OpenStack Share API v1.
 """
 
-from oslo_log import log
-
 from manila.api import extensions
 import manila.api.openstack
-from manila.api.v1 import cgsnapshots
-from manila.api.v1 import consistency_groups
 from manila.api.v1 import limits
 from manila.api.v1 import scheduler_stats
 from manila.api.v1 import security_service
-from manila.api.v1 import share_instances
+from manila.api.v1 import share_manage
 from manila.api.v1 import share_metadata
 from manila.api.v1 import share_networks
 from manila.api.v1 import share_servers
 from manila.api.v1 import share_snapshots
-from manila.api.v1 import share_types
+from manila.api.v1 import share_types_extra_specs
+from manila.api.v1 import share_unmanage
 from manila.api.v1 import shares
+from manila.api.v2 import availability_zones
+from manila.api.v2 import quota_class_sets
+from manila.api.v2 import quota_sets
+from manila.api.v2 import services
+from manila.api.v2 import share_types
 from manila.api import versions
 
-LOG = log.getLogger(__name__)
-
 
 class APIRouter(manila.api.openstack.APIRouter):
     """Route API requests.
@@ -56,24 +56,46 @@ class APIRouter(manila.api.openstack.APIRouter):
 
         mapper.redirect("", "/")
 
+        self.resources["availability_zones"] = (
+            availability_zones.create_resource_legacy())
+        mapper.resource("availability-zone",
+                        "os-availability-zone",
+                        controller=self.resources["availability_zones"])
+
+        self.resources["services"] = services.create_resource_legacy()
+        mapper.resource("service",
+                        "os-services",
+                        controller=self.resources["services"])
+
+        self.resources["quota_sets"] = quota_sets.create_resource_legacy()
+        mapper.resource("quota-set",
+                        "os-quota-sets",
+                        controller=self.resources["quota_sets"],
+                        member={'defaults': 'GET'})
+
+        self.resources["quota_class_sets"] = (
+            quota_class_sets.create_resource_legacy())
+        mapper.resource("quota-class-set",
+                        "os-quota-class-sets",
+                        controller=self.resources["quota_class_sets"])
+
+        self.resources["share_manage"] = share_manage.create_resource()
+        mapper.resource("share_manage",
+                        "os-share-manage",
+                        controller=self.resources["share_manage"])
+
+        self.resources["share_unmanage"] = share_unmanage.create_resource()
+        mapper.resource("share_unmanage",
+                        "os-share-unmanage",
+                        controller=self.resources["share_unmanage"],
+                        member={'unmanage': 'POST'})
+
         self.resources['shares'] = shares.create_resource()
         mapper.resource("share", "shares",
                         controller=self.resources['shares'],
                         collection={'detail': 'GET'},
                         member={'action': 'POST'})
 
-        self.resources['share_instances'] = share_instances.create_resource()
-        mapper.resource("share_instance", "share_instances",
-                        controller=self.resources['share_instances'],
-                        collection={'detail': 'GET'},
-                        member={'action': 'POST'})
-
-        mapper.connect("share_instance",
-                       "/{project_id}/shares/{share_id}/instances",
-                       controller=self.resources['share_instances'],
-                       action='get_share_instances',
-                       conditions={"method": ['GET']})
-
         self.resources['snapshots'] = share_snapshots.create_resource()
         mapper.resource("snapshot", "snapshots",
                         controller=self.resources['snapshots'],
@@ -112,7 +134,7 @@ class APIRouter(manila.api.openstack.APIRouter):
                         member={'action': 'POST'})
 
         self.resources['share_servers'] = share_servers.create_resource()
-        mapper.resource(share_servers.RESOURCE_NAME,
+        mapper.resource('share_server',
                         'share-servers',
                         controller=self.resources['share_servers'])
         mapper.connect('details',
@@ -125,7 +147,15 @@ class APIRouter(manila.api.openstack.APIRouter):
         mapper.resource("type", "types",
                         controller=self.resources['types'],
                         collection={'detail': 'GET', 'default': 'GET'},
-                        member={'action': 'POST'})
+                        member={'action': 'POST',
+                                'os-share-type-access': 'GET'})
+
+        self.resources['extra_specs'] = (
+            share_types_extra_specs.create_resource())
+        mapper.resource('extra_spec', 'extra_specs',
+                        controller=self.resources['extra_specs'],
+                        parent_resource=dict(member_name='type',
+                                             collection_name='types'))
 
         self.resources['scheduler_stats'] = scheduler_stats.create_resource()
         mapper.connect('pools', '/{project_id}/scheduler-stats/pools',
@@ -136,20 +166,3 @@ class APIRouter(manila.api.openstack.APIRouter):
                        controller=self.resources['scheduler_stats'],
                        action='pools_detail',
                        conditions={'method': ['GET']})
-
-        self.resources['consistency-groups'] = (
-            consistency_groups.create_resource())
-        mapper.resource('consistency-group', 'consistency-groups',
-                        controller=self.resources['consistency-groups'],
-                        collection={'detail': 'GET'})
-        mapper.connect('consistency-groups',
-                       '/{project_id}/consistency-groups/{id}/action',
-                       controller=self.resources['consistency-groups'],
-                       action='action',
-                       conditions={"action": ['POST']})
-
-        self.resources['cgsnapshots'] = cgsnapshots.create_resource()
-        mapper.resource('cgsnapshot', 'cgsnapshots',
-                        controller=self.resources['cgsnapshots'],
-                        collection={'detail': 'GET'},
-                        member={'members': 'GET', 'action': 'POST'})
