@@ -43,7 +43,6 @@ these objects be simple dictionaries.
 from oslo_config import cfg
 from oslo_db import api as db_api
 
-
 db_opts = [
     cfg.StrOpt('db_backend',
                default='sqlalchemy',
@@ -68,7 +67,19 @@ IMPL = db_api.DBAPI.from_config(cfg.CONF, backend_mapping=_BACKEND_MAPPING,
                                 lazy=True)
 
 
+def authorize_project_context(context, project_id):
+    """Ensures a request has permission to access the given project."""
+    return IMPL.authorize_project_context(context, project_id)
+
+
+def authorize_quota_class_context(context, class_name):
+    """Ensures a request has permission to access the given quota class."""
+    return IMPL.authorize_quota_class_context(context, class_name)
+
+
 ###################
+
+
 def service_destroy(context, service_id):
     """Destroy the service or raise if it does not exist."""
     return IMPL.service_destroy(context, service_id)
@@ -327,7 +338,7 @@ def share_instances_get_all_by_share_network(context, share_network_id):
 
 def share_instances_get_all_by_share(context, share_id):
     """Returns list of shares that belong to given share."""
-    return IMPL.share_instances_get_all_by_share_network(context, share_id)
+    return IMPL.share_instances_get_all_by_share(context, share_id)
 
 
 def share_instances_get_all_by_consistency_group_id(context, cg_id):
@@ -383,14 +394,6 @@ def share_get_all_by_consistency_group_id(context, cg_id,
         sort_key=sort_key, sort_dir=sort_dir)
 
 
-def share_get_all_by_share_network(context, share_network_id, filters=None,
-                                   sort_key=None, sort_dir=None):
-    """Returns list of shares that belong to given share network."""
-    return IMPL.share_get_all_by_share_network(
-        context, share_network_id, filters=filters, sort_key=sort_key,
-        sort_dir=sort_dir)
-
-
 def share_get_all_by_share_server(context, share_server_id, filters=None,
                                   sort_key=None, sort_dir=None):
     """Returns all shares with given share server ID."""
@@ -413,6 +416,15 @@ def share_access_create(context, values):
     return IMPL.share_access_create(context, values)
 
 
+def share_instance_access_copy(context, share_id, instance_id):
+    """Maps the existing access rules for the share to the instance in the DB.
+
+    Adds the instance mapping to the share's access rules and
+    returns the share's access rules.
+    """
+    return IMPL.share_instance_access_copy(context, share_id, instance_id)
+
+
 def share_access_get(context, access_id):
     """Get share access rule."""
     return IMPL.share_access_get(context, access_id)
@@ -433,6 +445,12 @@ def share_instance_access_get_all(context, access_id, session=None):
     return IMPL.share_instance_access_get_all(context, access_id, session=None)
 
 
+def share_access_get_all_for_instance(context, instance_id, session=None):
+    """Get all access rules related to a certain share instance."""
+    return IMPL.share_access_get_all_for_instance(
+        context, instance_id, session=None)
+
+
 def share_access_get_all_by_type_and_access(context, share_id, access_type,
                                             access):
     """Returns share access by given type and access."""
@@ -450,28 +468,47 @@ def share_instance_access_delete(context, mapping_id):
     return IMPL.share_instance_access_delete(context, mapping_id)
 
 
-def share_instance_access_update_state(context, mapping_id, state):
-    """Update state of access rule mapping."""
-    return IMPL.share_instance_access_update_state(context, mapping_id, state)
+def share_instance_update_access_status(context, share_instance_id, status):
+    """Update access rules status of share instance."""
+    return IMPL.share_instance_update_access_status(context, share_instance_id,
+                                                    status)
 
 
 ####################
 
 
 def share_snapshot_instance_update(context, instance_id, values):
-    """Set the given properties on an snapshot instance and update it.
+    """Set the given properties on a share snapshot instance and update it.
 
     Raises NotFound if snapshot instance does not exist.
     """
     return IMPL.share_snapshot_instance_update(context, instance_id, values)
 
 
+def share_snapshot_instance_create(context, snapshot_id, values):
+    """Create a share snapshot instance for an existing snapshot."""
+    return IMPL.share_snapshot_instance_create(
+        context, snapshot_id, values)
+
+
 def share_snapshot_instance_get(context, instance_id, with_share_data=False):
-    """Get a snapshot or raise if it does not exist."""
+    """Get a snapshot instance or raise a NotFound exception."""
     return IMPL.share_snapshot_instance_get(
         context, instance_id, with_share_data=with_share_data)
 
 
+def share_snapshot_instance_get_all_with_filters(context, filters,
+                                                 with_share_data=False):
+    """Get all snapshot instances satisfying provided filters."""
+    return IMPL.share_snapshot_instance_get_all_with_filters(
+        context, filters, with_share_data=with_share_data)
+
+
+def share_snapshot_instance_delete(context, snapshot_instance_id):
+    """Delete a share snapshot instance."""
+    return IMPL.share_snapshot_instance_delete(context, snapshot_instance_id)
+
+
 ####################
 
 
@@ -587,20 +624,61 @@ def share_metadata_update(context, share, metadata, delete):
 
 ###################
 
+def share_export_location_get_by_uuid(context, export_location_uuid):
+    """Get specific export location of a share."""
+    return IMPL.share_export_location_get_by_uuid(
+        context, export_location_uuid)
+
+
 def share_export_locations_get(context, share_id):
-    """Get all exports_locations of share."""
+    """Get all export locations of a share."""
     return IMPL.share_export_locations_get(context, share_id)
 
 
+def share_export_locations_get_by_share_id(context, share_id,
+                                           include_admin_only=True):
+    """Get all export locations of a share by its ID."""
+    return IMPL.share_export_locations_get_by_share_id(
+        context, share_id, include_admin_only=include_admin_only)
+
+
+def share_export_locations_get_by_share_instance_id(context,
+                                                    share_instance_id):
+    """Get all export locations of a share instance by its ID."""
+    return IMPL.share_export_locations_get_by_share_instance_id(
+        context, share_instance_id)
+
+
 def share_export_locations_update(context, share_instance_id, export_locations,
                                   delete=True):
-    """Update export locations of share."""
-    return IMPL.share_export_locations_update(context, share_instance_id,
-                                              export_locations, delete)
+    """Update export locations of a share instance."""
+    return IMPL.share_export_locations_update(
+        context, share_instance_id, export_locations, delete)
 
 
 ####################
 
+def export_location_metadata_get(context, export_location_uuid, session=None):
+    """Get all metadata of an export location."""
+    return IMPL.export_location_metadata_get(
+        context, export_location_uuid, session=session)
+
+
+def export_location_metadata_delete(context, export_location_uuid, keys,
+                                    session=None):
+    """Delete metadata of an export location."""
+    return IMPL.export_location_metadata_delete(
+        context, export_location_uuid, keys, session=session)
+
+
+def export_location_metadata_update(context, export_location_uuid, metadata,
+                                    delete, session=None):
+    """Update metadata of an export location."""
+    return IMPL.export_location_metadata_update(
+        context, export_location_uuid, metadata, delete, session=session)
+
+####################
+
 
 def share_network_create(context, values):
     """Create a share network DB record."""
@@ -650,6 +728,9 @@ def share_network_remove_security_service(context, id, security_service_id):
                                                       security_service_id)
 
 
+##################
+
+
 def network_allocation_create(context, values):
     """Create a network allocation DB record."""
     return IMPL.network_allocation_create(context, values)
@@ -666,11 +747,10 @@ def network_allocation_update(context, id, values):
 
 
 def network_allocations_get_for_share_server(context, share_server_id,
-                                             session=None):
-    """Get network allocation for share server."""
-    return IMPL.network_allocations_get_for_share_server(context,
-                                                         share_server_id,
-                                                         session=session)
+                                             session=None, label=None):
+    """Get network allocations for share server."""
+    return IMPL.network_allocations_get_for_share_server(
+        context, share_server_id, label=label, session=session)
 
 
 def network_allocations_get_by_ip_address(context, ip_address):
@@ -701,20 +781,6 @@ def share_server_get(context, id, session=None):
     return IMPL.share_server_get(context, id, session=session)
 
 
-def share_server_get_by_host(context, host, share_net_id, session=None):
-    """Get share server DB records by host."""
-    return IMPL.share_server_get_by_host(context, host, share_net_id,
-                                         session=session)
-
-
-def share_server_get_by_host_and_share_net(context, host, share_net_id,
-                                           session=None):
-    """Get share server DB records by host and share net."""
-    return IMPL.share_server_get_by_host_and_share_net(context, host,
-                                                       share_net_id,
-                                                       session=session)
-
-
 def share_server_get_all_by_host_and_share_net_valid(context, host,
                                                      share_net_id,
                                                      session=None):
@@ -803,11 +869,6 @@ def share_type_access_remove(context, type_id, project_id):
     return IMPL.share_type_access_remove(context, type_id, project_id)
 
 
-def share_type_qos_specs_get(context, type_id):
-    """Get all qos specs for given share type."""
-    return IMPL.share_type_qos_specs_get(context, type_id)
-
-
 def share_type_destroy(context, id):
     """Delete a share type."""
     return IMPL.share_type_destroy(context, id)
@@ -898,6 +959,12 @@ def consistency_group_create(context, values):
     return IMPL.consistency_group_create(context, values)
 
 
+def consistency_group_get_all_by_share_server(context, share_server_id):
+    """Get all consistency groups associated with a share server."""
+    return IMPL.consistency_group_get_all_by_share_server(context,
+                                                          share_server_id)
+
+
 def consistency_group_get_all_by_project(context, project_id, detailed=True):
     """Get all consistency groups belonging to a project."""
     return IMPL.consistency_group_get_all_by_project(context, project_id,
@@ -991,3 +1058,59 @@ def cgsnapshot_member_update(context, member_id, values):
     Raises NotFound if cgsnapshot member does not exist.
     """
     return IMPL.cgsnapshot_member_update(context, member_id, values)
+
+
+####################
+
+def share_replicas_get_all(context, with_share_server=False,
+                           with_share_data=False):
+    """Returns all share replicas regardless of share."""
+    return IMPL.share_replicas_get_all(
+        context, with_share_server=with_share_server,
+        with_share_data=with_share_data)
+
+
+def share_replicas_get_all_by_share(context, share_id, with_share_server=False,
+                                    with_share_data=False):
+    """Returns all share replicas for a given share."""
+    return IMPL.share_replicas_get_all_by_share(
+        context, share_id, with_share_server=with_share_server,
+        with_share_data=with_share_data)
+
+
+def share_replicas_get_available_active_replica(context, share_id,
+                                                with_share_server=False,
+                                                with_share_data=False):
+    """Returns an active replica for a given share."""
+    return IMPL.share_replicas_get_available_active_replica(
+        context, share_id, with_share_server=with_share_server,
+        with_share_data=with_share_data)
+
+
+def share_replicas_get_active_replicas_by_share(context, share_id,
+                                                with_share_server=False,
+                                                with_share_data=False):
+    """Returns all active replicas for a given share."""
+    return IMPL.share_replicas_get_active_replicas_by_share(
+        context, share_id, with_share_server=with_share_server,
+        with_share_data=with_share_data)
+
+
+def share_replica_get(context, replica_id, with_share_server=False,
+                      with_share_data=False):
+    """Get share replica by id."""
+    return IMPL.share_replica_get(
+        context, replica_id, with_share_server=with_share_server,
+        with_share_data=with_share_data)
+
+
+def share_replica_update(context, share_replica_id, values,
+                         with_share_data=False):
+    """Updates a share replica with given values."""
+    return IMPL.share_replica_update(context, share_replica_id, values,
+                                     with_share_data=with_share_data)
+
+
+def share_replica_delete(context, share_replica_id):
+    """Deletes a share replica."""
+    return IMPL.share_replica_delete(context, share_replica_id)
