@@ -39,17 +39,18 @@ import manila.network.neutron.neutron_network_plugin
 import manila.network.nova_network_plugin
 import manila.network.standalone_network_plugin
 import manila.quota
-import manila.scheduler.driver
+import manila.scheduler.drivers.base
+import manila.scheduler.drivers.simple
 import manila.scheduler.host_manager
 import manila.scheduler.manager
 import manila.scheduler.scheduler_options
-import manila.scheduler.simple
-import manila.scheduler.weights
-import manila.scheduler.weights.capacity
-import manila.scheduler.weights.pool
+import manila.scheduler.weighers
+import manila.scheduler.weighers.capacity
+import manila.scheduler.weighers.pool
 import manila.service
 import manila.share.api
 import manila.share.driver
+import manila.share.drivers.cephfs.cephfs_native
 import manila.share.drivers.emc.driver
 import manila.share.drivers.emc.plugins.isilon.isilon
 import manila.share.drivers.generic
@@ -60,14 +61,16 @@ import manila.share.drivers.glusterfs.layout_directory
 import manila.share.drivers.glusterfs.layout_volume
 import manila.share.drivers.hdfs.hdfs_native
 import manila.share.drivers.hitachi.hds_hnas
-import manila.share.drivers.hp.hp_3par_driver
+import manila.share.drivers.hpe.hpe_3par_driver
 import manila.share.drivers.huawei.huawei_nas
 import manila.share.drivers.ibm.gpfs
 import manila.share.drivers.netapp.options
 import manila.share.drivers.quobyte.quobyte
 import manila.share.drivers.service_instance
+import manila.share.drivers.tegile.tegile
 import manila.share.drivers.windows.service_instance
 import manila.share.drivers.windows.winrm_helper
+import manila.share.drivers.zfsonlinux.driver
 import manila.share.drivers.zfssa.zfssashare
 import manila.share.drivers_private_data
 import manila.share.hook
@@ -99,19 +102,20 @@ _global_opt_lists = [
     manila.network.nova_network_plugin.nova_single_network_plugin_opts,
     manila.network.standalone_network_plugin.standalone_network_plugin_opts,
     manila.quota.quota_opts,
-    manila.scheduler.driver.scheduler_driver_opts,
+    manila.scheduler.drivers.base.scheduler_driver_opts,
     manila.scheduler.host_manager.host_manager_opts,
     [manila.scheduler.manager.scheduler_driver_opt],
     [manila.scheduler.scheduler_options.scheduler_json_config_location_opt],
-    manila.scheduler.simple.simple_scheduler_opts,
-    manila.scheduler.weights.capacity.capacity_weight_opts,
-    manila.scheduler.weights.pool.pool_weight_opts,
+    manila.scheduler.drivers.simple.simple_scheduler_opts,
+    manila.scheduler.weighers.capacity.capacity_weight_opts,
+    manila.scheduler.weighers.pool.pool_weight_opts,
     manila.service.service_opts,
     manila.share.api.share_api_opts,
     manila.share.driver.ganesha_opts,
     manila.share.driver.share_opts,
     manila.share.driver.ssh_opts,
     manila.share.drivers_private_data.private_data_opts,
+    manila.share.drivers.cephfs.cephfs_native.cephfs_native_opts,
     manila.share.drivers.emc.driver.EMC_NAS_OPTS,
     manila.share.drivers.generic.share_opts,
     manila.share.drivers.glusterfs.common.glusterfs_common_opts,
@@ -122,7 +126,7 @@ _global_opt_lists = [
     manila.share.drivers.glusterfs.layout_volume.glusterfs_volume_mapped_opts,
     manila.share.drivers.hdfs.hdfs_native.hdfs_native_share_opts,
     manila.share.drivers.hitachi.hds_hnas.hds_hnas_opts,
-    manila.share.drivers.hp.hp_3par_driver.HP3PAR_OPTS,
+    manila.share.drivers.hpe.hpe_3par_driver.HPE3PAR_OPTS,
     manila.share.drivers.huawei.huawei_nas.huawei_opts,
     manila.share.drivers.ibm.gpfs.gpfs_share_opts,
     manila.share.drivers.netapp.options.netapp_proxy_opts,
@@ -130,12 +134,15 @@ _global_opt_lists = [
     manila.share.drivers.netapp.options.netapp_transport_opts,
     manila.share.drivers.netapp.options.netapp_basicauth_opts,
     manila.share.drivers.netapp.options.netapp_provisioning_opts,
+    manila.share.drivers.netapp.options.netapp_replication_opts,
     manila.share.drivers.quobyte.quobyte.quobyte_manila_share_opts,
     manila.share.drivers.service_instance.common_opts,
     manila.share.drivers.service_instance.no_share_servers_handling_mode_opts,
     manila.share.drivers.service_instance.share_servers_handling_mode_opts,
+    manila.share.drivers.tegile.tegile.tegile_opts,
     manila.share.drivers.windows.service_instance.windows_share_server_opts,
     manila.share.drivers.windows.winrm_helper.winrm_opts,
+    manila.share.drivers.zfsonlinux.driver.zfsonlinux_opts,
     manila.share.drivers.zfssa.zfssashare.ZFSSA_OPTS,
     manila.share.hook.hook_options,
     manila.share.manager.share_manager_opts,
@@ -153,6 +160,9 @@ _opts.extend(oslo_concurrency.opts.list_opts())
 _opts.extend(oslo_log._options.list_opts())
 _opts.extend(oslo_middleware.opts.list_opts())
 _opts.extend(oslo_policy.opts.list_opts())
+_opts.extend(manila.network.neutron.api.list_opts())
+_opts.extend(manila.compute.nova.list_opts())
+_opts.extend(manila.volume.cinder.list_opts())
 
 
 def list_opts():
