@@ -26,7 +26,8 @@ from oslo_log import log
 from oslo_utils import excutils
 
 from manila import exception
-from manila.i18n import _, _LE, _LW
+from manila.i18n import _, _LE, _LW, _LI
+from manila.share.drivers.netapp.dataontap.client import client_cmode
 from manila.share.drivers.netapp.dataontap.cluster_mode import lib_base
 from manila.share.drivers.netapp import utils as na_utils
 from manila import utils
@@ -34,6 +35,7 @@ from manila import utils
 
 LOG = log.getLogger(__name__)
 SUPPORTED_NETWORK_TYPES = (None, 'flat', 'vlan')
+SEGMENTED_NETWORK_TYPES = ('vlan',)
 
 
 class NetAppCmodeMultiSVMFileStorageLibrary(
@@ -89,6 +91,7 @@ class NetAppCmodeMultiSVMFileStorageLibrary(
     def _handle_housekeeping_tasks(self):
         """Handle various cleanup activities."""
         self._client.prune_deleted_nfs_export_policies()
+        self._client.prune_deleted_snapshots()
 
         super(NetAppCmodeMultiSVMFileStorageLibrary, self).\
             _handle_housekeeping_tasks()
@@ -111,7 +114,7 @@ class NetAppCmodeMultiSVMFileStorageLibrary(
         server_details = {'vserver_name': vserver_name}
 
         try:
-            self._create_vserver_if_nonexistent(vserver_name, network_info)
+            self._create_vserver(vserver_name, network_info)
         except Exception as e:
             e.detail_data = {'server_details': server_details}
             raise
@@ -132,58 +135,107 @@ class NetAppCmodeMultiSVMFileStorageLibrary(
         return self.configuration.netapp_vserver_name_template % server_id
 
     @na_utils.trace
-    def _create_vserver_if_nonexistent(self, vserver_name, network_info):
+    def _create_vserver(self, vserver_name, network_info):
         """Creates Vserver with given parameters if it doesn't exist."""
 
         if self._client.vserver_exists(vserver_name):
             msg = _('Vserver %s already exists.')
             raise exception.NetAppException(msg % vserver_name)
 
+        ipspace_name = self._create_ipspace(network_info)
+
         LOG.debug('Vserver %s does not exist, creating.', vserver_name)
         self._client.create_vserver(
             vserver_name,
             self.configuration.netapp_root_volume_aggregate,
             self.configuration.netapp_root_volume,
-            self._find_matching_aggregates())
+            self._find_matching_aggregates(),
+            ipspace_name)
 
         vserver_client = self._get_api_client(vserver=vserver_name)
+        security_services = None
         try:
             self._create_vserver_lifs(vserver_name,
                                       vserver_client,
-                                      network_info)
+                                      network_info,
+                                      ipspace_name)
+
+            self._create_vserver_admin_lif(vserver_name,
+                                           vserver_client,
+                                           network_info,
+                                           ipspace_name)
+
+            vserver_client.enable_nfs()
+
+            security_services = network_info.get('security_services')
+            if security_services:
+                self._client.setup_security_services(security_services,
+                                                     vserver_client,
+                                                     vserver_name)
         except Exception:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE("Failed to create network interface(s)."))
-                self._client.delete_vserver(vserver_name, vserver_client)
+                LOG.error(_LE("Failed to configure Vserver."))
+                self._delete_vserver(vserver_name,
+                                     security_services=security_services)
 
-        vserver_client.enable_nfs()
-
-        security_services = network_info.get('security_services')
-        if security_services:
-            self._client.setup_security_services(security_services,
-                                                 vserver_client,
-                                                 vserver_name)
+    def _get_valid_ipspace_name(self, network_id):
+        """Get IPspace name according to network id."""
+        return 'ipspace_' + network_id.replace('-', '_')
 
     @na_utils.trace
-    def _create_vserver_lifs(self, vserver_name, vserver_client,
-                             network_info):
+    def _create_ipspace(self, network_info):
+        """If supported, create an IPspace for a new Vserver."""
+
+        if not self._client.features.IPSPACES:
+            return None
+
+        if (network_info['network_allocations'][0]['network_type']
+                not in SEGMENTED_NETWORK_TYPES):
+            return client_cmode.DEFAULT_IPSPACE
+
+        # NOTE(cknight): Neutron needs cDOT IP spaces because it can provide
+        # overlapping IP address ranges for different subnets.  That is not
+        # believed to be an issue for any of Manila's other network plugins.
+        ipspace_id = network_info.get('neutron_subnet_id')
+        if not ipspace_id:
+            return client_cmode.DEFAULT_IPSPACE
+
+        ipspace_name = self._get_valid_ipspace_name(ipspace_id)
+        if not self._client.ipspace_exists(ipspace_name):
+            self._client.create_ipspace(ipspace_name)
+
+        return ipspace_name
+
+    @na_utils.trace
+    def _create_vserver_lifs(self, vserver_name, vserver_client, network_info,
+                             ipspace_name):
+        """Create Vserver data logical interfaces (LIFs)."""
 
         nodes = self._client.list_cluster_nodes()
         node_network_info = zip(nodes, network_info['network_allocations'])
-        netmask = utils.cidr_to_netmask(network_info['cidr'])
 
-        for node, net_info in node_network_info:
-            net_id = net_info['id']
-            port = self._get_node_data_port(node)
-            ip = net_info['ip_address']
-            self._create_lif_if_nonexistent(vserver_name,
-                                            net_id,
-                                            network_info['segmentation_id'],
-                                            node,
-                                            port,
-                                            ip,
-                                            netmask,
-                                            vserver_client)
+        for node_name, network_allocation in node_network_info:
+            lif_name = self._get_lif_name(node_name, network_allocation)
+            self._create_lif(vserver_client, vserver_name, ipspace_name,
+                             node_name, lif_name, network_allocation)
+
+    @na_utils.trace
+    def _create_vserver_admin_lif(self, vserver_name, vserver_client,
+                                  network_info, ipspace_name):
+        """Create Vserver admin LIF, if defined."""
+
+        network_allocations = network_info.get('admin_network_allocations')
+        if not network_allocations:
+            LOG.info(_LI('No admin network defined for Vserver %s.') %
+                     vserver_name)
+            return
+
+        node_name = self._client.list_cluster_nodes()[0]
+        network_allocation = network_allocations[0]
+        lif_name = self._get_lif_name(node_name, network_allocation)
+
+        self._create_lif(vserver_client, vserver_name, ipspace_name,
+                         node_name, lif_name, network_allocation)
 
     @na_utils.trace
     def _get_node_data_port(self, node):
@@ -197,22 +249,41 @@ class NetAppCmodeMultiSVMFileStorageLibrary(
                   'to create Vserver LIFs.') % node)
         return matched_port_names[0]
 
+    def _get_lif_name(self, node_name, network_allocation):
+        """Get LIF name based on template from manila.conf file."""
+        lif_name_args = {
+            'node': node_name,
+            'net_allocation_id': network_allocation['id'],
+        }
+        return self.configuration.netapp_lif_name_template % lif_name_args
+
     @na_utils.trace
-    def _create_lif_if_nonexistent(self, vserver_name, allocation_id, vlan,
-                                   node, port, ip, netmask, vserver_client):
+    def _create_lif(self, vserver_client, vserver_name, ipspace_name,
+                    node_name, lif_name, network_allocation):
         """Creates LIF for Vserver."""
-        if not vserver_client.network_interface_exists(vserver_name, node,
-                                                       port, ip, netmask,
-                                                       vlan):
+
+        port = self._get_node_data_port(node_name)
+        ip_address = network_allocation['ip_address']
+        netmask = utils.cidr_to_netmask(network_allocation['cidr'])
+        vlan = network_allocation['segmentation_id']
+
+        if not vserver_client.network_interface_exists(
+                vserver_name, node_name, port, ip_address, netmask, vlan):
+
             self._client.create_network_interface(
-                ip, netmask, vlan, node, port, vserver_name, allocation_id,
-                self.configuration.netapp_lif_name_template)
+                ip_address, netmask, vlan, node_name, port, vserver_name,
+                lif_name, ipspace_name)
 
     @na_utils.trace
     def get_network_allocations_number(self):
         """Get number of network interfaces to be created."""
         return len(self._client.list_cluster_nodes())
 
+    @na_utils.trace
+    def get_admin_network_allocations_number(self, admin_network_api):
+        """Get number of network allocations for creating admin LIFs."""
+        return 1 if admin_network_api else 0
+
     @na_utils.trace
     def teardown_server(self, server_details, security_services=None):
         """Teardown share server."""
@@ -231,7 +302,19 @@ class NetAppCmodeMultiSVMFileStorageLibrary(
                             "record will proceed anyway."), vserver)
             return
 
+        self._delete_vserver(vserver, security_services=security_services)
+
+    @na_utils.trace
+    def _delete_vserver(self, vserver, security_services=None):
+        """Delete a Vserver plus IPspace and security services as needed."""
+
+        ipspace_name = self._client.get_vserver_ipspace(vserver)
+
         vserver_client = self._get_api_client(vserver=vserver)
         self._client.delete_vserver(vserver,
                                     vserver_client,
                                     security_services=security_services)
+
+        if ipspace_name and not self._client.ipspace_has_data_vservers(
+                ipspace_name):
+            self._client.delete_ipspace(ipspace_name)
