@@ -17,6 +17,7 @@ import ddt
 import mock
 import netaddr
 from oslo_config import cfg
+import six
 
 from manila.common import constants
 from manila import context
@@ -66,7 +67,7 @@ class StandaloneNetworkPluginTest(test.TestCase):
                 'standalone_network_plugin_gateway': '10.0.0.1',
                 'standalone_network_plugin_mask': '255.255.0.0',
                 'standalone_network_plugin_network_type': 'vlan',
-                'standalone_network_plugin_segmentation_id': '1001',
+                'standalone_network_plugin_segmentation_id': 1001,
                 'standalone_network_plugin_allowed_ip_ranges': (
                     '10.0.0.3-10.0.0.7,10.0.0.69-10.0.0.157,10.0.0.213'),
                 'standalone_network_plugin_ip_version': 4,
@@ -85,7 +86,7 @@ class StandaloneNetworkPluginTest(test.TestCase):
         self.assertEqual('10.0.0.1', instance.gateway)
         self.assertEqual('255.255.0.0', instance.mask)
         self.assertEqual('vlan', instance.network_type)
-        self.assertEqual('1001', instance.segmentation_id)
+        self.assertEqual(1001, instance.segmentation_id)
         self.assertEqual(allowed_cidrs, instance.allowed_cidrs)
         self.assertEqual(
             ['10.0.0.3-10.0.0.7', '10.0.0.69-10.0.0.157', '10.0.0.213'],
@@ -133,7 +134,7 @@ class StandaloneNetworkPluginTest(test.TestCase):
                 'standalone_network_plugin_gateway': '2001:db8::0001',
                 'standalone_network_plugin_mask': '88',
                 'standalone_network_plugin_network_type': 'vlan',
-                'standalone_network_plugin_segmentation_id': '3999',
+                'standalone_network_plugin_segmentation_id': 3999,
                 'standalone_network_plugin_allowed_ip_ranges': (
                     '2001:db8::-2001:db8:0000:0000:0000:007f:ffff:ffff'),
                 'standalone_network_plugin_ip_version': 6,
@@ -147,7 +148,7 @@ class StandaloneNetworkPluginTest(test.TestCase):
         self.assertEqual('2001:db8::0001', instance.gateway)
         self.assertEqual('88', instance.mask)
         self.assertEqual('vlan', instance.network_type)
-        self.assertEqual('3999', instance.segmentation_id)
+        self.assertEqual(3999, instance.segmentation_id)
         self.assertEqual(['2001:db8::/89'], instance.allowed_cidrs)
         self.assertEqual(
             ['2001:db8::-2001:db8:0000:0000:0000:007f:ffff:ffff'],
@@ -165,7 +166,7 @@ class StandaloneNetworkPluginTest(test.TestCase):
                 'standalone_network_plugin_gateway': '10.0.0.1',
                 'standalone_network_plugin_mask': '255.255.0.0',
                 'standalone_network_plugin_network_type': network_type,
-                'standalone_network_plugin_segmentation_id': '1001',
+                'standalone_network_plugin_segmentation_id': 1001,
                 'standalone_network_plugin_ip_version': 4,
             },
         }
@@ -183,7 +184,7 @@ class StandaloneNetworkPluginTest(test.TestCase):
                 'standalone_network_plugin_gateway': '10.0.0.1',
                 'standalone_network_plugin_mask': '255.255.0.0',
                 'standalone_network_plugin_network_type': fake_network_type,
-                'standalone_network_plugin_segmentation_id': '1001',
+                'standalone_network_plugin_segmentation_id': 1001,
                 'standalone_network_plugin_ip_version': 4,
             },
         }
@@ -307,7 +308,7 @@ class StandaloneNetworkPluginTest(test.TestCase):
         instance.db.share_network_update.assert_called_once_with(
             fake_context, fake_share_network['id'],
             dict(network_type=None, segmentation_id=None,
-                 cidr=instance.net.cidr, ip_version=4))
+                 cidr=six.text_type(instance.net.cidr), ip_version=4))
 
     def test_allocate_network_zero_addresses_ipv6(self):
         data = {
@@ -328,13 +329,13 @@ class StandaloneNetworkPluginTest(test.TestCase):
         instance.db.share_network_update.assert_called_once_with(
             fake_context, fake_share_network['id'],
             dict(network_type=None, segmentation_id=None,
-                 cidr=instance.net.cidr, ip_version=6))
+                 cidr=six.text_type(instance.net.cidr), ip_version=6))
 
     def test_allocate_network_one_ip_address_ipv4_no_usages_exist(self):
         data = {
             'DEFAULT': {
                 'standalone_network_plugin_network_type': 'vlan',
-                'standalone_network_plugin_segmentation_id': '1003',
+                'standalone_network_plugin_segmentation_id': 1003,
                 'standalone_network_plugin_gateway': '10.0.0.1',
                 'standalone_network_plugin_mask': '24',
             },
@@ -351,16 +352,21 @@ class StandaloneNetworkPluginTest(test.TestCase):
             fake_context, fake_share_server, fake_share_network)
 
         self.assertEqual(1, len(allocations))
+        na_data = {
+            'network_type': 'vlan',
+            'segmentation_id': 1003,
+            'cidr': '10.0.0.0/24',
+            'ip_version': 4,
+        }
         instance.db.share_network_update.assert_called_once_with(
-            fake_context, fake_share_network['id'],
-            dict(network_type='vlan', segmentation_id='1003',
-                 cidr=instance.net.cidr, ip_version=4))
+            fake_context, fake_share_network['id'], na_data)
         instance.db.network_allocations_get_by_ip_address.assert_has_calls(
             [mock.call(fake_context, '10.0.0.2')])
         instance.db.network_allocation_create.assert_called_once_with(
             fake_context,
             dict(share_server_id=fake_share_server['id'],
-                 ip_address='10.0.0.2', status=constants.STATUS_ACTIVE))
+                 ip_address='10.0.0.2', status=constants.STATUS_ACTIVE,
+                 label='user', **na_data))
 
     def test_allocate_network_two_ip_addresses_ipv4_two_usages_exist(self):
         ctxt = type('FakeCtxt', (object,), {'fake': ['10.0.0.2', '10.0.0.4']})
@@ -390,10 +396,14 @@ class StandaloneNetworkPluginTest(test.TestCase):
             ctxt, fake_share_server, fake_share_network, count=2)
 
         self.assertEqual(2, len(allocations))
+        na_data = {
+            'network_type': None,
+            'segmentation_id': None,
+            'cidr': six.text_type(instance.net.cidr),
+            'ip_version': 4,
+        }
         instance.db.share_network_update.assert_called_once_with(
-            ctxt, fake_share_network['id'],
-            dict(network_type=None, segmentation_id=None,
-                 cidr=instance.net.cidr, ip_version=4))
+            ctxt, fake_share_network['id'], dict(**na_data))
         instance.db.network_allocations_get_by_ip_address.assert_has_calls(
             [mock.call(ctxt, '10.0.0.2'), mock.call(ctxt, '10.0.0.3'),
              mock.call(ctxt, '10.0.0.4'), mock.call(ctxt, '10.0.0.5')])
@@ -401,11 +411,13 @@ class StandaloneNetworkPluginTest(test.TestCase):
             mock.call(
                 ctxt,
                 dict(share_server_id=fake_share_server['id'],
-                     ip_address='10.0.0.3', status=constants.STATUS_ACTIVE)),
+                     ip_address='10.0.0.3', status=constants.STATUS_ACTIVE,
+                     label='user', **na_data)),
             mock.call(
                 ctxt,
                 dict(share_server_id=fake_share_server['id'],
-                     ip_address='10.0.0.5', status=constants.STATUS_ACTIVE)),
+                     ip_address='10.0.0.5', status=constants.STATUS_ACTIVE,
+                     label='user', **na_data)),
         ])
 
     def test_allocate_network_no_available_ipv4_addresses(self):
@@ -431,6 +443,6 @@ class StandaloneNetworkPluginTest(test.TestCase):
         instance.db.share_network_update.assert_called_once_with(
             fake_context, fake_share_network['id'],
             dict(network_type=None, segmentation_id=None,
-                 cidr=instance.net.cidr, ip_version=4))
+                 cidr=six.text_type(instance.net.cidr), ip_version=4))
         instance.db.network_allocations_get_by_ip_address.assert_has_calls(
             [mock.call(fake_context, '10.0.0.2')])
