@@ -52,7 +52,7 @@ class SchedulerRpcAPITestCase(test.TestCase):
 
         def _fake_prepare_method(*args, **kwds):
             for kwd in kwds:
-                self.assertEqual(kwds[kwd], target[kwd])
+                self.assertEqual(target[kwd], kwds[kwd])
             return rpcapi.client
 
         def _fake_rpc_method(*args, **kwargs):
@@ -67,10 +67,10 @@ class SchedulerRpcAPITestCase(test.TestCase):
             with mock.patch.object(rpcapi.client, rpc_method) as mock_method:
                 mock_method.side_effect = _fake_rpc_method
                 retval = getattr(rpcapi, method)(ctxt, **kwargs)
-                self.assertEqual(retval, expected_retval)
+                self.assertEqual(expected_retval, retval)
                 expected_args = [ctxt, method, expected_msg]
                 for arg, expected_arg in zip(self.fake_args, expected_args):
-                    self.assertEqual(arg, expected_arg)
+                    self.assertEqual(expected_arg, arg)
 
     def test_update_service_capabilities(self):
         self._test_scheduler_api('update_service_capabilities',
@@ -103,10 +103,27 @@ class SchedulerRpcAPITestCase(test.TestCase):
 
     def test_migrate_share_to_host(self):
         self._test_scheduler_api('migrate_share_to_host',
-                                 rpc_method='cast',
+                                 rpc_method='call',
                                  share_id='share_id',
                                  host='host',
                                  force_host_copy=True,
+                                 notify=True,
                                  request_spec='fake_request_spec',
                                  filter_properties='filter_properties',
                                  version='1.4')
+
+    def test_create_share_replica(self):
+        self._test_scheduler_api('create_share_replica',
+                                 rpc_method='cast',
+                                 request_spec='fake_request_spec',
+                                 filter_properties='filter_properties',
+                                 version='1.5')
+
+    def test_manage_share(self):
+        self._test_scheduler_api('manage_share',
+                                 rpc_method='call',
+                                 share_id='share_id',
+                                 driver_options='fake_driver_options',
+                                 request_spec='fake_request_spec',
+                                 filter_properties='filter_properties',
+                                 version='1.6')
