@@ -60,6 +60,10 @@ def glusterXMLOut(**kwargs):
     return template % kwargs, ''
 
 
+FAKE_UUID1 = '11111111-1111-1111-1111-111111111111'
+FAKE_UUID2 = '22222222-2222-2222-2222-222222222222'
+
+
 @ddt.ddt
 class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
     """Tests GlusterfsVolumeMappedLayout."""
@@ -153,37 +157,88 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
 
         self.assertEqual(re.compile(volume_pattern), ret)
 
-    def test_fetch_gluster_volumes(self):
+    @ddt.data({'root@host1:/manila-share-1-1G': 'NONE',
+               'root@host2:/manila-share-2-2G': None},
+              {'root@host1:/manila-share-1-1G': FAKE_UUID1,
+               'root@host2:/manila-share-2-2G': None},
+              {'root@host1:/manila-share-1-1G': 'foobarbaz',
+               'root@host2:/manila-share-2-2G': FAKE_UUID2},
+              {'root@host1:/manila-share-1-1G': FAKE_UUID1,
+               'root@host2:/manila-share-2-2G': FAKE_UUID2})
+    def test_fetch_gluster_volumes(self, sharemark):
+        vol1_qualified = 'root@host1:/manila-share-1-1G'
+        gmgr_vol1 = common.GlusterManager(vol1_qualified)
+        gmgr_vol1.get_vol_option = mock.Mock(
+            return_value=sharemark[vol1_qualified])
+        vol2_qualified = 'root@host2:/manila-share-2-2G'
+        gmgr_vol2 = common.GlusterManager(vol2_qualified)
+        gmgr_vol2.get_vol_option = mock.Mock(
+            return_value=sharemark[vol2_qualified])
         self.mock_object(
             self.gmgr1, 'gluster_call',
             mock.Mock(return_value=(self.glusterfs_server1_volumes, '')))
         self.mock_object(
             self.gmgr2, 'gluster_call',
             mock.Mock(return_value=(self.glusterfs_server2_volumes, '')))
-        _glustermanager_calls = (
-            self.gmgr1,
-            common.GlusterManager('root@host1:/manila-share-1-1G'),
-            self.gmgr2,
-            common.GlusterManager('root@host2:/manila-share-2-2G'))
+        _glustermanager_calls = (self.gmgr1, gmgr_vol1, self.gmgr2, gmgr_vol2)
         self.mock_object(self._layout, '_glustermanager',
                          mock.Mock(side_effect=_glustermanager_calls))
-        expected_output = self.glusterfs_volumes_dict
+        expected_output = {}
+        for q, d in self.glusterfs_volumes_dict.items():
+            if sharemark[q] not in (FAKE_UUID1, FAKE_UUID2):
+                expected_output[q] = d
 
         ret = self._layout._fetch_gluster_volumes()
 
         test_args = ('volume', 'list')
-        self.gmgr1.gluster_call.assert_called_once_with(*test_args)
-        self.gmgr2.gluster_call.assert_called_once_with(*test_args)
+        self.gmgr1.gluster_call.assert_called_once_with(*test_args,
+                                                        log=mock.ANY)
+        self.gmgr2.gluster_call.assert_called_once_with(*test_args,
+                                                        log=mock.ANY)
+        gmgr_vol1.get_vol_option.assert_called_once_with(
+            'user.manila-share')
+        gmgr_vol2.get_vol_option.assert_called_once_with(
+            'user.manila-share')
+        self.assertEqual(expected_output, ret)
+
+    def test_fetch_gluster_volumes_no_filter_used(self):
+        vol1_qualified = 'root@host1:/manila-share-1-1G'
+        gmgr_vol1 = common.GlusterManager(vol1_qualified)
+        gmgr_vol1.get_vol_option = mock.Mock()
+        vol2_qualified = 'root@host2:/manila-share-2-2G'
+        gmgr_vol2 = common.GlusterManager(vol2_qualified)
+        gmgr_vol2.get_vol_option = mock.Mock()
+        self.mock_object(
+            self.gmgr1, 'gluster_call',
+            mock.Mock(return_value=(self.glusterfs_server1_volumes, '')))
+        self.mock_object(
+            self.gmgr2, 'gluster_call',
+            mock.Mock(return_value=(self.glusterfs_server2_volumes, '')))
+        _glustermanager_calls = (self.gmgr1, gmgr_vol1, self.gmgr2, gmgr_vol2)
+        self.mock_object(self._layout, '_glustermanager',
+                         mock.Mock(side_effect=_glustermanager_calls))
+        expected_output = self.glusterfs_volumes_dict
+
+        ret = self._layout._fetch_gluster_volumes(filter_used=False)
+
+        test_args = ('volume', 'list')
+        self.gmgr1.gluster_call.assert_called_once_with(*test_args,
+                                                        log=mock.ANY)
+        self.gmgr2.gluster_call.assert_called_once_with(*test_args,
+                                                        log=mock.ANY)
+        self.assertFalse(gmgr_vol1.get_vol_option.called)
+        self.assertFalse(gmgr_vol2.get_vol_option.called)
         self.assertEqual(expected_output, ret)
 
     def test_fetch_gluster_volumes_no_keymatch(self):
+        vol1_qualified = 'root@host1:/manila-share-1'
+        gmgr_vol1 = common.GlusterManager(vol1_qualified)
+        gmgr_vol1.get_vol_option = mock.Mock(return_value=None)
         self._layout.configuration.glusterfs_servers = [self.glusterfs_server1]
         self.mock_object(
             self.gmgr1, 'gluster_call',
             mock.Mock(return_value=('manila-share-1', '')))
-        _glustermanager_calls = (
-            self.gmgr1,
-            common.GlusterManager('root@host1:/manila-share-1'))
+        _glustermanager_calls = (self.gmgr1, gmgr_vol1)
         self.mock_object(self._layout, '_glustermanager',
                          mock.Mock(side_effect=_glustermanager_calls))
         self.mock_object(self._layout, 'volume_pattern',
@@ -193,7 +248,8 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         ret = self._layout._fetch_gluster_volumes()
 
         test_args = ('volume', 'list')
-        self.gmgr1.gluster_call.assert_called_once_with(*test_args)
+        self.gmgr1.gluster_call.assert_called_once_with(*test_args,
+                                                        log=mock.ANY)
         self.assertEqual(expected_output, ret)
 
     def test_fetch_gluster_volumes_error(self):
@@ -201,7 +257,7 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
 
         def raise_exception(*args, **kwargs):
             if(args == test_args):
-                raise exception.ProcessExecutionError()
+                raise exception.GlusterfsException()
 
         self._layout.configuration.glusterfs_servers = [self.glusterfs_server1]
         self.mock_object(self.gmgr1, 'gluster_call',
@@ -213,8 +269,8 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         self.assertRaises(exception.GlusterfsException,
                           self._layout._fetch_gluster_volumes)
 
-        self.gmgr1.gluster_call.assert_called_once_with(*test_args)
-        self.assertTrue(layout_volume.LOG.error.called)
+        self.gmgr1.gluster_call.assert_called_once_with(*test_args,
+                                                        log=mock.ANY)
 
     def test_do_setup(self):
         self._layout.configuration.glusterfs_servers = [self.glusterfs_server1]
@@ -226,11 +282,12 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
                          mock.Mock(return_value=self.glusterfs_volumes_dict))
         self.mock_object(self._layout, '_check_mount_glusterfs')
         self._layout.gluster_used_vols = self.glusterfs_used_vols
-        self.mock_object(layout_volume.LOG, 'warn')
+        self.mock_object(layout_volume.LOG, 'warning')
 
         self._layout.do_setup(self._context)
 
-        self._layout._fetch_gluster_volumes.assert_called_once_with()
+        self._layout._fetch_gluster_volumes.assert_called_once_with(
+            filter_used=False)
         self._layout._check_mount_glusterfs.assert_called_once_with()
         self.gmgr1.get_gluster_version.assert_called_once_with()
 
@@ -273,7 +330,8 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         self.assertRaises(exception.GlusterfsException,
                           self._layout.do_setup, self._context)
 
-        self._layout._fetch_gluster_volumes.assert_called_once_with()
+        self._layout._fetch_gluster_volumes.assert_called_once_with(
+            filter_used=False)
 
     def test_share_manager(self):
         self.mock_object(self._layout, '_glustermanager',
@@ -288,10 +346,21 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         self._layout._glustermanager.assert_called_once_with('host1:/gv1')
         self.assertEqual(self.gmgr1, ret)
 
+    def test_share_manager_no_privdata(self):
+        self.mock_object(self._layout.private_storage,
+                         'get', mock.Mock(return_value=None))
+
+        ret = self._layout._share_manager(self.share1)
+
+        self._layout.private_storage.get.assert_called_once_with(
+            self.share1['id'], 'volume')
+        self.assertEqual(None, ret)
+
     def test_ensure_share(self):
         share = self.share1
         gmgr1 = common.GlusterManager(self.glusterfs_target1, self._execute,
                                       None, None)
+        gmgr1.set_vol_option = mock.Mock()
         self.mock_object(self._layout, '_share_manager',
                          mock.Mock(return_value=gmgr1))
 
@@ -299,6 +368,8 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
 
         self._layout._share_manager.assert_called_once_with(share)
         self.assertIn(self.glusterfs_target1, self._layout.gluster_used_vols)
+        gmgr1.set_vol_option.assert_called_once_with(
+            'user.manila-share', share['id'])
 
     @ddt.data({"voldict": {"host:/share2G": {"size": 2}}, "used_vols": set(),
                "size": 1, "expected": "host:/share2G"},
@@ -478,8 +549,10 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
     def test_create_share(self):
         self._layout._pop_gluster_vol = mock.Mock(
             return_value=self.glusterfs_target1)
+        gmgr1 = common.GlusterManager(self.glusterfs_target1)
+        gmgr1.set_vol_option = mock.Mock()
         self.mock_object(self._layout, '_glustermanager',
-                         mock.Mock(return_value=self.gmgr1))
+                         mock.Mock(return_value=gmgr1))
         self.mock_object(self.fake_driver, '_setup_via_manager',
                          mock.Mock(return_value='host1:/gv1'))
 
@@ -488,9 +561,11 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
 
         self._layout._pop_gluster_vol.assert_called_once_with(share['size'])
         self.fake_driver._setup_via_manager.assert_called_once_with(
-            {'manager': self.gmgr1, 'share': share})
+            {'manager': gmgr1, 'share': share})
         self._layout.private_storage.update.assert_called_once_with(
             share['id'], {'volume': self.glusterfs_target1})
+        gmgr1.set_vol_option.assert_called_once_with(
+            'user.manila-share', share['id'])
         self.assertEqual('host1:/gv1', exp_locn)
 
     def test_create_share_error(self):
@@ -504,22 +579,52 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         self._layout._pop_gluster_vol.assert_called_once_with(
             share['size'])
 
-    def test_delete_share(self):
+    @ddt.data(None, '', 'Eeyore')
+    def test_delete_share(self, clone_of):
         self._layout._push_gluster_vol = mock.Mock()
         self._layout._wipe_gluster_vol = mock.Mock()
         gmgr = common.GlusterManager
         gmgr1 = gmgr(self.glusterfs_target1, self._execute, None, None)
+        gmgr1.set_vol_option = mock.Mock()
+        gmgr1.get_vol_option = mock.Mock(return_value=clone_of)
         self.mock_object(self._layout, '_glustermanager',
                          mock.Mock(return_value=gmgr1))
         self._layout.gluster_used_vols = set([self.glusterfs_target1])
 
         self._layout.delete_share(self._context, self.share1)
 
+        gmgr1.get_vol_option.assert_called_once_with(
+            'user.manila-cloned-from')
         self._layout._wipe_gluster_vol.assert_called_once_with(gmgr1)
         self._layout._push_gluster_vol.assert_called_once_with(
             self.glusterfs_target1)
         self._layout.private_storage.delete.assert_called_once_with(
             self.share1['id'])
+        gmgr1.set_vol_option.assert_called_once_with(
+            'user.manila-share', 'NONE')
+
+    def test_delete_share_clone(self):
+        self._layout._push_gluster_vol = mock.Mock()
+        self._layout._wipe_gluster_vol = mock.Mock()
+        gmgr = common.GlusterManager
+        gmgr1 = gmgr(self.glusterfs_target1, self._execute, None, None)
+        gmgr1.gluster_call = mock.Mock()
+        gmgr1.get_vol_option = mock.Mock(return_value=FAKE_UUID1)
+        self.mock_object(self._layout, '_glustermanager',
+                         mock.Mock(return_value=gmgr1))
+        self._layout.gluster_used_vols = set([self.glusterfs_target1])
+
+        self._layout.delete_share(self._context, self.share1)
+
+        gmgr1.get_vol_option.assert_called_once_with(
+            'user.manila-cloned-from')
+        self.assertFalse(self._layout._wipe_gluster_vol.called)
+        self._layout._push_gluster_vol.assert_called_once_with(
+            self.glusterfs_target1)
+        self._layout.private_storage.delete.assert_called_once_with(
+            self.share1['id'])
+        gmgr1.gluster_call.assert_called_once_with(
+            'volume', 'delete', 'gv1')
 
     def test_delete_share_error(self):
         self._layout._wipe_gluster_vol = mock.Mock()
@@ -528,6 +633,7 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         self._layout._push_gluster_vol = mock.Mock()
         gmgr = common.GlusterManager
         gmgr1 = gmgr(self.glusterfs_target1, self._execute, None, None)
+        gmgr1.get_vol_option = mock.Mock(return_value=None)
         self.mock_object(self._layout, '_glustermanager',
                          mock.Mock(return_value=gmgr1))
         self._layout.gluster_used_vols = set([self.glusterfs_target1])
@@ -539,6 +645,14 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         self._layout._wipe_gluster_vol.assert_called_once_with(gmgr1)
         self.assertFalse(self._layout._push_gluster_vol.called)
 
+    def test_delete_share_missing_record(self):
+        self.mock_object(self._layout, '_share_manager',
+                         mock.Mock(return_value=None))
+
+        self._layout.delete_share(self._context, self.share1)
+
+        self._layout._share_manager.assert_called_once_with(self.share1)
+
     def test_create_snapshot(self):
         self._layout.gluster_nosnap_vols_dict = {}
         self._layout.glusterfs_versions = {self.glusterfs_server1: ('3', '6')}
@@ -561,13 +675,10 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         self.assertIsNone(ret)
         args = ('--xml', 'snapshot', 'create', 'manila-fake_snap_id',
                 gmgr1.volume)
-        gmgr1.gluster_call.assert_called_once_with(*args)
+        gmgr1.gluster_call.assert_called_once_with(*args, log=mock.ANY)
 
     @ddt.data({'side_effect': (glusterXMLOut(ret=-1, errno=2),),
                '_exception': exception.GlusterfsException},
-              {'side_effect': exception.ProcessExecutionError,
-               '_exception': exception.GlusterfsException},
-              {'side_effect': RuntimeError, '_exception': RuntimeError},
               {'side_effect': (('', ''),),
                '_exception': exception.GlusterfsException})
     @ddt.unpack
@@ -592,7 +703,7 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
 
         args = ('--xml', 'snapshot', 'create', 'manila-fake_snap_id',
                 gmgr1.volume)
-        gmgr1.gluster_call.assert_called_once_with(*args)
+        gmgr1.gluster_call.assert_called_once_with(*args, log=mock.ANY)
 
     @ddt.data({"vers_minor": '6', "exctype": exception.GlusterfsException},
               {"vers_minor": '7',
@@ -621,7 +732,7 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
 
         args = ('--xml', 'snapshot', 'create', 'manila-fake_snap_id',
                 gmgr1.volume)
-        gmgr1.gluster_call.assert_called_once_with(*args)
+        gmgr1.gluster_call.assert_called_once_with(*args, log=mock.ANY)
 
     @ddt.data({"vers_minor": '6', "exctype": exception.GlusterfsException},
               {"vers_minor": '7',
@@ -660,25 +771,9 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         ret = self._layout._find_actual_backend_snapshot_name(gmgr1, snapshot)
 
         args = ('snapshot', 'list', gmgr1.volume, '--mode=script')
-        gmgr1.gluster_call.assert_called_once_with(*args)
+        gmgr1.gluster_call.assert_called_once_with(*args, log=mock.ANY)
         self.assertEqual('fake_snap_id_xyz', ret)
 
-    @ddt.data(exception.ProcessExecutionError, RuntimeError)
-    def test_find_actual_backend_snapshot_name_gluster_error(self, _exception):
-        gmgr = common.GlusterManager
-        gmgr1 = gmgr(self.share1['export_location'], self._execute, None, None)
-        self.mock_object(gmgr1, 'gluster_call',
-                         mock.Mock(side_effect=_exception))
-
-        self.assertRaises({exception.ProcessExecutionError:
-                           exception.GlusterfsException}.get(_exception,
-                                                             _exception),
-                          self._layout._find_actual_backend_snapshot_name,
-                          gmgr1, mock.Mock())
-
-        args = ('snapshot', 'list', gmgr1.volume, '--mode=script')
-        gmgr1.gluster_call.assert_called_once_with(*args)
-
     @ddt.data('this is too bad', 'fake_snap_id_xyx\nfake_snap_id_pqr')
     def test_find_actual_backend_snapshot_name_bad_snap_list(self, snaplist):
         gmgr = common.GlusterManager
@@ -696,7 +791,7 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
                           gmgr1, snapshot)
 
         args = ('snapshot', 'list', gmgr1.volume, '--mode=script')
-        gmgr1.gluster_call.assert_called_once_with(*args)
+        gmgr1.gluster_call.assert_called_once_with(*args, log=mock.ANY)
 
     @ddt.data({'glusterfs_target': 'root@host1:/gv1',
                'glusterfs_server': 'root@host1'},
@@ -708,7 +803,8 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         share = new_share()
         snapshot = {
             'id': 'fake_snap_id',
-            'share_instance': new_share(export_location=glusterfs_target)
+            'share_instance': new_share(export_location=glusterfs_target),
+            'share_id': 'fake_share_id',
         }
         volume = ''.join(['manila-', share['id']])
         new_vol_addr = ':/'.join([glusterfs_server, volume])
@@ -720,10 +816,10 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         self.mock_object(old_gmgr, 'gluster_call',
                          mock.Mock(side_effect=[('', ''), ('', '')]))
         self.mock_object(new_gmgr, 'gluster_call',
-                         mock.Mock(side_effect=[('', '')]))
-        self.mock_object(new_gmgr, 'get_gluster_vol_option',
+                         mock.Mock(side_effect=[('', ''), ('', ''), ('', '')]))
+        self.mock_object(new_gmgr, 'get_vol_option',
                          mock.Mock())
-        new_gmgr.get_gluster_vol_option.return_value = (
+        new_gmgr.get_vol_option.return_value = (
             'glusterfs-server-1,client')
         self.mock_object(self._layout, '_find_actual_backend_snapshot_name',
                          mock.Mock(return_value='fake_snap_id_xyz'))
@@ -742,9 +838,14 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         args = (('snapshot', 'activate', 'fake_snap_id_xyz',
                  'force', '--mode=script'),
                 ('snapshot', 'clone', volume, 'fake_snap_id_xyz'))
-        old_gmgr.gluster_call.assert_has_calls([mock.call(*a) for a in args])
-        args = (('volume', 'start', volume),)
-        new_gmgr.gluster_call.assert_has_calls([mock.call(*a) for a in args])
+        old_gmgr.gluster_call.assert_has_calls(
+            [mock.call(*a, log=mock.ANY) for a in args])
+        args = (('volume', 'start', volume),
+                ('volume', 'set', volume, 'user.manila-share', share['id']),
+                ('volume', 'set', volume, 'user.manila-cloned-from',
+                 snapshot['share_id']))
+        new_gmgr.gluster_call.assert_has_calls(
+            [mock.call(*a, log=mock.ANY) for a in args], any_order=True)
         self._layout._share_manager.assert_called_once_with(
             snapshot['share_instance'])
         self._layout._glustermanager.assert_called_once_with(
@@ -759,114 +860,6 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
             self._layout.gluster_used_vols)
         self.assertEqual('host1:/gv1', ret)
 
-    @ddt.data({'trouble': exception.ProcessExecutionError,
-               '_exception': exception.GlusterfsException},
-              {'trouble': RuntimeError, '_exception': RuntimeError})
-    @ddt.unpack
-    def test_create_share_from_snapshot_error_new_gmr_gluster_calls(
-            self, trouble, _exception):
-        glusterfs_target = 'root@host1:/gv1'
-        glusterfs_server = 'root@host1'
-        share = new_share()
-        volume = ''.join(['manila-', share['id']])
-        new_vol_addr = ':/'.join([glusterfs_server, volume])
-        gmgr = common.GlusterManager
-        old_gmgr = gmgr(glusterfs_target, self._execute, None, None)
-        new_gmgr = gmgr(new_vol_addr, self._execute, None, None)
-        self._layout.gluster_used_vols = set([glusterfs_target])
-        self._layout.glusterfs_versions = {glusterfs_server: ('3', '7')}
-        self.mock_object(old_gmgr, 'gluster_call',
-                         mock.Mock(side_effect=[('', ''), ('', '')]))
-        self.mock_object(new_gmgr, 'gluster_call',
-                         mock.Mock(side_effect=trouble))
-        self.mock_object(new_gmgr, 'get_gluster_vol_option',
-                         mock.Mock())
-        new_gmgr.get_gluster_vol_option.return_value = (
-            'glusterfs-server-1,client')
-        self.mock_object(self._layout, '_find_actual_backend_snapshot_name',
-                         mock.Mock(return_value='fake_snap_id_xyz'))
-        self.mock_object(self._layout, '_share_manager',
-                         mock.Mock(return_value=old_gmgr))
-        self.mock_object(self._layout, '_glustermanager',
-                         mock.Mock(return_value=new_gmgr))
-        self.mock_object(self.fake_driver, '_setup_via_manager',
-                         mock.Mock(return_value='host1:/gv1'))
-
-        snapshot = {
-            'id': 'fake_snap_id',
-            'share_instance': new_share(export_location=glusterfs_target)
-        }
-        self.assertRaises(_exception,
-                          self._layout.create_share_from_snapshot,
-                          self._context, share, snapshot)
-
-        (self._layout._find_actual_backend_snapshot_name.
-            assert_called_once_with(old_gmgr, snapshot))
-        args = (('snapshot', 'activate', 'fake_snap_id_xyz',
-                 'force', '--mode=script'),
-                ('snapshot', 'clone', volume, 'fake_snap_id_xyz'))
-        old_gmgr.gluster_call.assert_has_calls([mock.call(*a) for a in args])
-        args = (('volume', 'start', volume),)
-        new_gmgr.gluster_call.assert_has_calls([mock.call(*a) for a in args])
-        self._layout._share_manager.assert_called_once_with(
-            snapshot['share_instance'])
-        self._layout._glustermanager.assert_called_once_with(
-            gmgr.parse(new_vol_addr))
-        self._layout.driver._setup_via_manager.assert_called_once_with(
-            {'manager': new_gmgr, 'share': share},
-            {'manager': old_gmgr, 'share': snapshot['share_instance']})
-
-    @ddt.data({'trouble': exception.ProcessExecutionError,
-               '_exception': exception.GlusterfsException},
-              {'trouble': RuntimeError, '_exception': RuntimeError})
-    @ddt.unpack
-    def test_create_share_from_snapshot_error_old_gmr_gluster_calls(
-            self, trouble, _exception):
-        glusterfs_target = 'root@host1:/gv1'
-        glusterfs_server = 'root@host1'
-        share = new_share()
-        volume = ''.join(['manila-', share['id']])
-        new_vol_addr = ':/'.join([glusterfs_server, volume])
-        gmgr = common.GlusterManager
-        old_gmgr = gmgr(glusterfs_target, self._execute, None, None)
-        new_gmgr = gmgr(new_vol_addr, self._execute, None, None)
-        self._layout.gluster_used_vols_dict = {glusterfs_target: old_gmgr}
-        self._layout.glusterfs_versions = {glusterfs_server: ('3', '7')}
-        self.mock_object(
-            old_gmgr, 'gluster_call',
-            mock.Mock(side_effect=[('', ''), trouble]))
-        self.mock_object(new_gmgr, 'get_gluster_vol_option',
-                         mock.Mock())
-        new_gmgr.get_gluster_vol_option.return_value = (
-            'glusterfs-server-1,client')
-        self.mock_object(self._layout, '_find_actual_backend_snapshot_name',
-                         mock.Mock(return_value='fake_snap_id_xyz'))
-        self.mock_object(self._layout, '_share_manager',
-                         mock.Mock(return_value=old_gmgr))
-        self.mock_object(self._layout, '_glustermanager',
-                         mock.Mock(return_value=new_gmgr))
-
-        snapshot = {
-            'id': 'fake_snap_id',
-            'share_instance': new_share(export_location=glusterfs_target)
-        }
-        self.assertRaises(_exception,
-                          self._layout.create_share_from_snapshot,
-                          self._context, share, snapshot)
-
-        (self._layout._find_actual_backend_snapshot_name.
-            assert_called_once_with(old_gmgr, snapshot))
-        args = (('snapshot', 'activate', 'fake_snap_id_xyz',
-                 'force', '--mode=script'),
-                ('snapshot', 'clone', volume, 'fake_snap_id_xyz'))
-        old_gmgr.gluster_call.assert_has_calls([mock.call(*a) for a in args])
-        self._layout._share_manager.assert_called_once_with(
-            snapshot['share_instance'])
-        self.assertFalse(new_gmgr.get_gluster_vol_option.called)
-        self.assertFalse(new_gmgr.gluster_call.called)
-        self.assertNotIn(new_vol_addr,
-                         self._layout.glusterfs_versions.keys())
-
     def test_create_share_from_snapshot_error_unsupported_gluster_version(
             self):
         glusterfs_target = 'root@host1:/gv1'
@@ -882,9 +875,9 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         self.mock_object(
             old_gmgr, 'gluster_call',
             mock.Mock(side_effect=[('', ''), ('', '')]))
-        self.mock_object(new_gmgr, 'get_gluster_vol_option',
+        self.mock_object(new_gmgr, 'get_vol_option',
                          mock.Mock())
-        new_gmgr.get_gluster_vol_option.return_value = (
+        new_gmgr.get_vol_option.return_value = (
             'glusterfs-server-1,client')
         self.mock_object(self._layout, '_find_actual_backend_snapshot_name',
                          mock.Mock(return_value='fake_snap_id_xyz'))
@@ -907,7 +900,7 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         self._layout._share_manager.assert_called_once_with(
             snapshot['share_instance'])
         self.assertFalse(self._layout._glustermanager.called)
-        self.assertFalse(new_gmgr.get_gluster_vol_option.called)
+        self.assertFalse(new_gmgr.get_vol_option.called)
         self.assertFalse(new_gmgr.gluster_call.called)
         self.assertNotIn(new_vol_addr,
                          self._layout.glusterfs_versions.keys())
@@ -934,15 +927,12 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         self.assertIsNone(ret)
         args = ('--xml', 'snapshot', 'delete', 'fake_snap_id_xyz',
                 '--mode=script')
-        gmgr1.gluster_call.assert_called_once_with(*args)
+        gmgr1.gluster_call.assert_called_once_with(*args, log=mock.ANY)
         (self._layout._find_actual_backend_snapshot_name.
             assert_called_once_with(gmgr1, snapshot))
 
-    @ddt.data({'side_effect': (glusterXMLOut(ret=-1, errno=2),),
+    @ddt.data({'side_effect': (glusterXMLOut(ret=-1, errno=0),),
                '_exception': exception.GlusterfsException},
-              {'side_effect': exception.ProcessExecutionError,
-               '_exception': exception.GlusterfsException},
-              {'side_effect': RuntimeError, '_exception': RuntimeError},
               {'side_effect': (('', ''),),
                '_exception': exception.GlusterfsException})
     @ddt.unpack
@@ -969,7 +959,7 @@ class GlusterfsVolumeMappedLayoutTestCase(test.TestCase):
         self.assertRaises(_exception, self._layout.delete_snapshot,
                           self._context, snapshot)
 
-        gmgr1.gluster_call.assert_called_once_with(*args)
+        gmgr1.gluster_call.assert_called_once_with(*args, log=mock.ANY)
         (self._layout._find_actual_backend_snapshot_name.
             assert_called_once_with(gmgr1, snapshot))
 
