@@ -58,43 +58,53 @@ class HackingTestCase(test.TestCase):
     """
 
     def test_no_translate_debug_logs(self):
-        self.assertEqual(len(list(checks.no_translate_debug_logs(
-            "LOG.debug(_('foo'))", "manila/scheduler/foo.py"))), 1)
+        self.assertEqual(1, len(list(checks.no_translate_debug_logs(
+            "LOG.debug(_('foo'))", "manila/scheduler/foo.py"))))
 
-        self.assertEqual(len(list(checks.no_translate_debug_logs(
-            "LOG.debug('foo')", "manila/scheduler/foo.py"))), 0)
+        self.assertEqual(0, len(list(checks.no_translate_debug_logs(
+            "LOG.debug('foo')", "manila/scheduler/foo.py"))))
 
-        self.assertEqual(len(list(checks.no_translate_debug_logs(
-            "LOG.info(_('foo'))", "manila/scheduler/foo.py"))), 0)
+        self.assertEqual(0, len(list(checks.no_translate_debug_logs(
+            "LOG.info(_('foo'))", "manila/scheduler/foo.py"))))
 
     def test_check_explicit_underscore_import(self):
-        self.assertEqual(len(list(checks.check_explicit_underscore_import(
+        self.assertEqual(1, len(list(checks.check_explicit_underscore_import(
             "LOG.info(_('My info message'))",
-            "cinder/tests/other_files.py"))), 1)
-        self.assertEqual(len(list(checks.check_explicit_underscore_import(
+            "cinder/tests/other_files.py"))))
+        self.assertEqual(1, len(list(checks.check_explicit_underscore_import(
             "msg = _('My message')",
-            "cinder/tests/other_files.py"))), 1)
-        self.assertEqual(len(list(checks.check_explicit_underscore_import(
+            "cinder/tests/other_files.py"))))
+        self.assertEqual(0, len(list(checks.check_explicit_underscore_import(
             "from cinder.i18n import _",
-            "cinder/tests/other_files.py"))), 0)
-        self.assertEqual(len(list(checks.check_explicit_underscore_import(
+            "cinder/tests/other_files.py"))))
+        self.assertEqual(0, len(list(checks.check_explicit_underscore_import(
             "LOG.info(_('My info message'))",
-            "cinder/tests/other_files.py"))), 0)
-        self.assertEqual(len(list(checks.check_explicit_underscore_import(
+            "cinder/tests/other_files.py"))))
+        self.assertEqual(0, len(list(checks.check_explicit_underscore_import(
             "msg = _('My message')",
-            "cinder/tests/other_files.py"))), 0)
-        self.assertEqual(len(list(checks.check_explicit_underscore_import(
-            "from cinder.i18n import _, _LW",
-            "cinder/tests/other_files2.py"))), 0)
-        self.assertEqual(len(list(checks.check_explicit_underscore_import(
+            "cinder/tests/other_files.py"))))
+        self.assertEqual(0, len(list(checks.check_explicit_underscore_import(
+            "from cinder.i18n import _LE, _, _LW",
+            "cinder/tests/other_files2.py"))))
+        self.assertEqual(0, len(list(checks.check_explicit_underscore_import(
             "msg = _('My message')",
-            "cinder/tests/other_files2.py"))), 0)
-        self.assertEqual(len(list(checks.check_explicit_underscore_import(
+            "cinder/tests/other_files2.py"))))
+        self.assertEqual(0, len(list(checks.check_explicit_underscore_import(
             "_ = translations.ugettext",
-            "cinder/tests/other_files3.py"))), 0)
-        self.assertEqual(len(list(checks.check_explicit_underscore_import(
+            "cinder/tests/other_files3.py"))))
+        self.assertEqual(0, len(list(checks.check_explicit_underscore_import(
             "msg = _('My message')",
-            "cinder/tests/other_files3.py"))), 0)
+            "cinder/tests/other_files3.py"))))
+        # Complete code coverage by falling through all checks
+        self.assertEqual(0, len(list(checks.check_explicit_underscore_import(
+            "LOG.info('My info message')",
+            "cinder.tests.unit/other_files4.py"))))
+        self.assertEqual(0, len(list(checks.check_explicit_underscore_import(
+            "from cinder.i18n import _LW",
+            "cinder.tests.unit/other_files5.py"))))
+        self.assertEqual(1, len(list(checks.check_explicit_underscore_import(
+            "msg = _('My message')",
+            "cinder.tests.unit/other_files5.py"))))
 
     # We are patching pep8 so that only the check under test is actually
     # installed.
@@ -200,3 +210,31 @@ class HackingTestCase(test.TestCase):
                """
         errors = []
         self._assert_has_errors(code, checker, expected_errors=errors)
+
+    def test_dict_constructor_with_list_copy(self):
+        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
+            "    dict([(i, connect_info[i])"))))
+
+        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
+            "    attrs = dict([(k, _from_json(v))"))))
+
+        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
+            "        type_names = dict((value, key) for key, value in"))))
+
+        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
+            "   dict((value, key) for key, value in"))))
+
+        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
+            "foo(param=dict((k, v) for k, v in bar.items()))"))))
+
+        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
+            " dict([[i,i] for i in range(3)])"))))
+
+        self.assertEqual(1, len(list(checks.dict_constructor_with_list_copy(
+            "  dd = dict([i,i] for i in range(3))"))))
+
+        self.assertEqual(0, len(list(checks.dict_constructor_with_list_copy(
+            "        create_kwargs = dict(snapshot=snapshot,"))))
+
+        self.assertEqual(0, len(list(checks.dict_constructor_with_list_copy(
+            "      self._render_dict(xml, data_el, data.__dict__)"))))
