@@ -14,13 +14,14 @@
 #    under the License.
 
 import six
-from tempest import config  # noqa
-from tempest import test  # noqa
-from tempest_lib.common.utils import data_utils  # noqa
-from tempest_lib import exceptions as lib_exc  # noqa
-import testtools  # noqa
+from tempest import config
+from tempest.lib.common.utils import data_utils
+from tempest.lib import exceptions as lib_exc
+from tempest import test
+import testtools
 
 from manila_tempest_tests.tests.api import base
+from manila_tempest_tests import utils
 
 CONF = config.CONF
 
@@ -75,32 +76,37 @@ class ManageNFSShareTest(base.BaseSharesAdminTest):
             'share_type_id': cls.st['share_type']['id'],
             'share_protocol': cls.protocol,
         }}
-        # Create two shares in parallel
-        cls.shares = cls.create_shares([creation_data, creation_data])
 
+        # Data for creating shares in parallel
+        data = [creation_data, creation_data]
+        if utils.is_microversion_ge(CONF.share.max_api_microversion, "2.8"):
+            data.append(creation_data)
+        shares_created = cls.create_shares(data)
+
+        cls.shares = []
         # Load all share data (host, etc.)
-        cls.share1 = cls.shares_client.get_share(cls.shares[0]['id'])
-        cls.share2 = cls.shares_client.get_share(cls.shares[1]['id'])
+        for share in shares_created:
+            # Unmanage shares from manila
+            cls.shares.append(cls.shares_client.get_share(share['id']))
+            cls.shares_client.unmanage_share(share['id'])
+            cls.shares_client.wait_for_resource_deletion(
+                share_id=share['id'])
 
-        # Unmanage shares from manila
-        cls.shares_client.unmanage_share(cls.share1['id'])
-        cls.shares_client.wait_for_resource_deletion(share_id=cls.share1['id'])
-        cls.shares_client.unmanage_share(cls.share2['id'])
-        cls.shares_client.wait_for_resource_deletion(share_id=cls.share2['id'])
-
-    @test.attr(type=["gate", "smoke"])
-    def test_manage(self):
-        name = "Name for 'managed' share that had ID %s" % self.share1["id"]
+    def _test_manage(self, share, is_public=False,
+                     version=CONF.share.max_api_microversion):
+        name = "Name for 'managed' share that had ID %s" % \
+               share['id']
         description = "Description for 'managed' share"
 
         # Manage share
-        share = self.shares_client.manage_share(
-            service_host=self.share1['host'],
-            export_path=self.share1['export_locations'][0],
-            protocol=self.share1['share_proto'],
+        share = self.shares_v2_client.manage_share(
+            service_host=share['host'],
+            export_path=share['export_locations'][0],
+            protocol=share['share_proto'],
             share_type_id=self.st['share_type']['id'],
             name=name,
             description=description,
+            is_public=is_public,
         )
 
         # Add managed share to cleanup queue
@@ -109,53 +115,72 @@ class ManageNFSShareTest(base.BaseSharesAdminTest):
                 'client': self.shares_client})
 
         # Wait for success
-        self.shares_client.wait_for_share_status(share['id'], 'available')
+        self.shares_v2_client.wait_for_share_status(share['id'], 'available')
 
         # Verify data of managed share
         get = self.shares_v2_client.get_share(share['id'], version="2.5")
         self.assertEqual(name, get['name'])
         self.assertEqual(description, get['description'])
-        self.assertEqual(self.share1['host'], get['host'])
-        self.assertEqual(self.share1['share_proto'], get['share_proto'])
+        self.assertEqual(share['host'], get['host'])
+        self.assertEqual(share['share_proto'], get['share_proto'])
         self.assertEqual(self.st['share_type']['name'], get['share_type'])
 
-        get = self.shares_v2_client.get_share(share['id'], version="2.6")
-        self.assertEqual(self.st['share_type']['id'], get['share_type'])
+        share = self.shares_v2_client.get_share(share['id'], version="2.6")
+        self.assertEqual(self.st['share_type']['id'], share['share_type'])
+
+        if utils.is_microversion_ge(version, "2.8"):
+            self.assertEqual(is_public, share['is_public'])
+        else:
+            self.assertFalse(share['is_public'])
 
         # Delete share
-        self.shares_client.delete_share(share['id'])
-        self.shares_client.wait_for_resource_deletion(share_id=share['id'])
+        self.shares_v2_client.delete_share(share['id'])
+        self.shares_v2_client.wait_for_resource_deletion(share_id=share['id'])
         self.assertRaises(lib_exc.NotFound,
-                          self.shares_client.get_share,
+                          self.shares_v2_client.get_share,
                           share['id'])
 
+    @base.skip_if_microversion_not_supported("2.8")
+    @test.attr(type=["gate", "smoke"])
+    def test_manage_with_is_public_True(self):
+        self._test_manage(share=self.shares[2], is_public=True)
+
+    @test.attr(type=["gate", "smoke"])
+    def test_manage(self):
+        # After 'unmanage' operation, share instance should be deleted.
+        # Assert not related to 'manage' test, but placed here for
+        # resource optimization.
+        share_instance_list = self.shares_v2_client.list_share_instances()
+        share_ids = [si['share_id'] for si in share_instance_list]
+        self.assertNotIn(self.shares[0]['id'], share_ids)
+
+        self._test_manage(share=self.shares[0])
+
     @test.attr(type=["gate", "smoke"])
     def test_manage_retry(self):
         # Manage share with invalid parameters
-        share = None
-        parameters = [(self.st_invalid['share_type']['id'], 'manage_error'),
-                      (self.st['share_type']['id'], 'available')]
 
-        for share_type_id, status in parameters:
-            share = self.shares_client.manage_share(
-                service_host=self.share2['host'],
-                export_path=self.share2['export_locations'][0],
-                protocol=self.share2['share_proto'],
-                share_type_id=share_type_id)
+        self.assertRaises(
+            lib_exc.Conflict,
+            self.shares_v2_client.manage_share,
+            service_host=self.shares[1]['host'],
+            export_path=self.shares[1]['export_locations'][0],
+            protocol=self.shares[1]['share_proto'],
+            share_type_id=self.st_invalid['share_type']['id'])
 
-            # Add managed share to cleanup queue
-            self.method_resources.insert(
-                0, {'type': 'share', 'id': share['id'],
-                    'client': self.shares_client})
+        share = self.shares_v2_client.manage_share(
+            service_host=self.shares[1]['host'],
+            export_path=self.shares[1]['export_locations'][0],
+            protocol=self.shares[1]['share_proto'],
+            share_type_id=self.st['share_type']['id'])
 
-            # Wait for success
-            self.shares_client.wait_for_share_status(share['id'], status)
+        self.shares_v2_client.wait_for_share_status(share['id'], 'available')
 
         # Delete share
-        self.shares_client.delete_share(share['id'])
-        self.shares_client.wait_for_resource_deletion(share_id=share['id'])
+        self.shares_v2_client.delete_share(share['id'])
+        self.shares_v2_client.wait_for_resource_deletion(share_id=share['id'])
         self.assertRaises(lib_exc.NotFound,
-                          self.shares_client.get_share,
+                          self.shares_v2_client.get_share,
                           share['id'])
 
 
@@ -169,3 +194,7 @@ class ManageGLUSTERFSShareTest(ManageNFSShareTest):
 
 class ManageHDFSShareTest(ManageNFSShareTest):
     protocol = 'hdfs'
+
+
+class ManageCephFSShareTest(ManageNFSShareTest):
+    protocol = 'cephfs'
