@@ -18,8 +18,8 @@ import six
 
 from tempest.common.utils.linux import remote_client  # noqa
 from tempest import config  # noqa
+from tempest.lib.common.utils import data_utils
 from tempest.scenario import manager  # noqa
-from tempest_lib.common.utils import data_utils
 
 from manila_tempest_tests import clients_share
 
@@ -38,6 +38,7 @@ class ShareScenarioTest(manager.NetworkScenarioTest):
 
         # Manila clients
         cls.shares_client = clients_share.Manager().shares_client
+        cls.shares_v2_client = clients_share.Manager().shares_v2_client
         cls.shares_admin_client = clients_share.AdminManager().shares_client
         cls.shares_admin_v2_client = (
             clients_share.AdminManager().shares_v2_client)
@@ -132,7 +133,12 @@ class ShareScenarioTest(manager.NetworkScenarioTest):
         """
         client = client or self.shares_client
         access = client.create_access_rule(share_id, access_type, access_to)
-        client.wait_for_access_rule_status(share_id, access['id'], "active")
+
+        # NOTE(u_glide): Ignore provided client, because we always need v2
+        # client to make this call
+        self.shares_v2_client.wait_for_share_status(
+            share_id, "active", status_attr='access_rules_status')
+
         if cleanup:
             self.addCleanup(client.delete_access_rule, share_id, access['id'])
         return access
@@ -151,10 +157,9 @@ class ShareScenarioTest(manager.NetworkScenarioTest):
             tenant_id = client.tenant_id
         if not router_id:
             router_id = self._get_router()['id']
-        client.add_router_interface_with_subnet_id(router_id,
-                                                   subnet_id)
-        self.addCleanup(client.remove_router_interface_with_subnet_id,
-                        router_id, subnet_id)
+        client.add_router_interface(router_id, subnet_id=subnet_id)
+        self.addCleanup(
+            client.remove_router_interface, router_id, subnet_id=subnet_id)
 
     def get_remote_client(self, *args, **kwargs):
         if not CONF.share.image_with_share_tools:
@@ -190,8 +195,9 @@ class ShareScenarioTest(manager.NetworkScenarioTest):
 
     def _migrate_share(self, share_id, dest_host, client=None):
         client = client or self.shares_admin_v2_client
-        client.migrate_share(share_id, dest_host)
-        share = client.wait_for_migration_completed(share_id, dest_host)
+        client.migrate_share(share_id, dest_host, True)
+        share = client.wait_for_migration_status(share_id, dest_host,
+                                                 'migration_success')
         return share
 
     def _create_share_type(self, name, is_public=True, **kwargs):
